#ffactor.r
#Andrew Campbell
#Marine Institite, Galway
#andrew.campbell@marine.ie

############################################################ 
#Function to calculate f factor from TAC####################
############################################################

ffactor<-function(tac,pop,fv,disf,wt,m,numage){

   #argument list
   #tac - the desired tac
   #pop - population number vector
   #fv - current fishing mortality vector
   #disf - discard mortality vector
   #wt - weight at age vector
   #m - natural mortality vector
   #numage - number of age groups

   #all arguments are mandatory
   #all arguments must be numeric
   #tac must be positive and of length 1
   #pop must be positive and of length = numage
   #fv must be of length numage
   #disf must be of length numage
   #wt must be positive and of length numage
   #m must be of length numage
   #numage must be a positive integer

   missing.args<-c()

   if (missing(tac)) missing.args<-c(missing.args,"tac")
   if (missing(pop)) missing.args<-c(missing.args,"pop")
   if (missing(fv)) missing.args<-c(missing.args,"fv")
   if (missing(disf)) missing.args<-c(missing.args,"disf")
   if (missing(wt)) missing.args<-c(missing.args,"wt")
   if (missing(m)) missing.args<-c(missing.args,"m")
   if (missing(numage)) missing.args<-c(missing.args,"numage")

   if (length(missing.args)>0) {
      stop(cat("ffactor::the following required argument(s) are missing:",missing.args,"\n"))
   }

   if (!(is.numeric(tac))) stop("ffactor::tac must be numeric\n")
   if (!(length(tac)==1)) stop("ffactor::tac must be of length 1\n")
   if (tac<0) stop("ffactor::tac must be >=0\n")

   if (!(is.numeric(numage))) stop("ffactor::numage must be numeric\n")
   if (!(length(numage)==1)) stop("ffactor::numage must be of length 1\n")
   if (!(numage>0)) stop("ffactor::numage must be greater than zero\n")

   if (!(is.numeric(pop))) stop("ffactor::pop must be numeric\n")
   if (!(length(pop)==numage)) stop(cat("ffactor::pop must be of length",numage,"\n"))
   if (sum(pop<0)>0) stop("ffactor::pop must be >=0\n")

   if (!(is.numeric(fv))) stop("ffactor::fv must be numeric\n")
   if (!(length(fv)==numage)) stop(cat("ffactor::fv must be of length",numage,"\n"))
   if (sum(fv<0)>0) stop("ffactor::fv must be >=0\n")

   if (!(is.numeric(disf))) stop("ffactor::disf must be numeric\n")
   if (!(length(disf)==numage)) stop(cat("ffactor::disf must be of length",numage,"\n"))
   if (sum(disf<0)>0) stop("ffactor::disf must be >=0\n")

   if (!(is.numeric(wt))) stop("ffactor::wt must be numeric\n")
   if (!(length(wt)==numage)) stop(cat("ffactor::wt must be of length",numage,"\n"))
   if (sum(wt<0)>0) stop("ffactor::wt must be >=0\n")

   if (!(is.numeric(m))) stop("ffactor::m must be numeric\n")
   if (!(length(m)==numage)) stop(cat("ffactor::m must be of length",numage,"\n"))
   if (sum(m<0)>0) stop("ffactor::m must be >=0\n")

   #ensure a notional minimum f value of 0.00001
   for (j in 1:numage) {fv[j] <- max(fv[j],0.00001)}

   f<-fv

   #current biomass vector
   biomass<-pop*wt

   #if the tac is zero then set the returned vector of ffactors to zero
   if (tac==0) {

      rat<-0

   } else {

      #if the tac is 95% (or greater) of the current biomass i.e. effectively all the population then
      #set the multiplier to 1000

      if ((0.95*sum(biomass))<tac) {

         rat<-1000

      } else {

         #factor is calculated by initialising it to 1, calculating the resulting catch and 
         #comparing it with the desired tac. If the difference is greater than 0.1% of the tac
         #then the multiplier is modified by a factor of the tac divided by the total catch weight.

	 #initialise vars
         x<-1            #used to test for convergence
         rat<-1          #the fishing mortality multiplier

         while (x>0.001) {

            #compute modified fishing mortality using the current multiplier value
            f<-rat*fv

            #compute the test catch
            test_catch<-f*biomass*(1-exp(-f-m-disf))/(disf+f+m+0.00001)

            #multiplier for the rat value is the ratio of the test catch to the desired tac
            rat<-rat*(tac/sum(test_catch))

            #if the total catch weight less the tac is within 0.1% of the desired tac then
            #we can assume we have converged
            x<-abs((sum(test_catch)-tac)/tac)

         }
      }
   }

   #return the multiplier
   ffac<-rat

   return(ffac)

}

