#hcr.r
#Andrew Campbell
#Marine Institite, Galway
#andrew.campbell@marine.ie
#HCR functions

############################################################ 
# HCR rule 1
# This function compares the simulated assessment estimate of SSB (oldssb) to 
# a fixed reference level ssbhcr. If oldssb > ssbhcr then a value of hcrmult = 1 
# is returned (virtual fishery is unchanged). If oldssb < ssbhcr then a value of 
# hcrmult = oldssb / ssbhcr (will always be <1) is returned (resulting in a 
# reduction in the virtual fishery relative to how far below the reference level 
# the estimate of the virtual SSB is).

hcr1<-function(oldssb,ssbhcr){

   #check for missing arguments
   if (missing(oldssb) & missing(ssbhcr)) stop("hcr1::oldssb and ssbhcr are mandatory arguments\n")
   if (missing(oldssb)) stop("hcr1::oldssb is a mandatory argument\n")
   if (missing(ssbhcr)) stop("hcr1::ssbhcr is a mandatory argument\n")

   #check arguments supplied are numeric
   if (!is.numeric(oldssb)) stop ("hcr1::argument oldssb must be numeric\n")
   if (!is.numeric(ssbhcr)) stop ("hcr1::argument ssbhcr must be numeric\n")
   if (oldssb<0) stop ("hcr1::argument oldssb must have a value greater than zero\n")
   if (ssbhcr<=0) stop ("hcr1::argument ssbhcr must have a value greater than zero\n")

   #default value
   hcrmulty <- 1

   #if ssb is less than ssbhcr, set the multiplier to oldssb/ssbhcr
   if (oldssb < ssbhcr) {
      hcrmulty <- oldssb/ssbhcr
   }

   return(c(hcrmulty,1))

}

############################################################ 
# HCR rule 2
# This function compares the simulated assessment estimate of SSB (oldssb) to 
# a fixed reference level ssbhcr. If oldssb > ssbhcr then a value of hcrmult = 1 
# is returned. If oldssb < ssbhcr then a value of hcrmult = 0 is returned 

hcr2<-function(oldssb,ssbhcr){

   #check for missing arguments
   if (missing(oldssb) & missing(ssbhcr)) stop("hcr2::oldssb and ssbhcr are mandatory arguments\n")
   if (missing(oldssb)) stop("hcr2::oldssb is a mandatory argument\n")
   if (missing(ssbhcr)) stop("hcr2::ssbhcr is a mandatory argument\n")

   #check arguments supplied are numeric
   if (!is.numeric(oldssb)) stop ("hcr2::argument oldssb must be numeric\n")
   if (!is.numeric(ssbhcr)) stop ("hcr2::argument ssbhcr must be numeric\n")
   if (oldssb<0) stop ("hcr2::argument oldssb must have a value greater than zero\n")
   if (ssbhcr<0) stop ("hcr2::argument ssbhcr must have a value greater than zero\n")

   #default value
   hcrmulty<-1

   #if ssb is less than ssbhcr, set the multiplier to 0 to close the fishery
   if (oldssb<ssbhcr) {
      hcrmulty<-0
   }

   return(c(hcrmulty,1))
  
}

############################################################ 
# HCR rule 3
# This function compares the simulated assessment estimate of SSB (oldssb) to 
# a fixed reference level ssbhcr. A value of hcrmult = oldssb/ssbhcr is always 
# saved and returned (resulting in an increase or reduction in the virtual fishery 
# depending on whether oldssb > ssbhcr).

hcr3<-function(oldssb,ssbhcr){

   #check for missing arguments
   if (missing(oldssb) & missing(ssbhcr)) stop("hcr3::oldssb and ssbhcr are mandatory arguments\n")
   if (missing(oldssb)) stop("hcr3::oldssb is a mandatory argument\n")
   if (missing(ssbhcr)) stop("hcr3::ssbhcr is a mandatory argument\n")

   #check arguments supplied are numeric
   if (!is.numeric(oldssb)) stop ("hcr3::argument oldssb must be numeric\n")
   if (!is.numeric(ssbhcr)) stop ("hcr3::argument ssbhcr must be numeric\n")
   if (oldssb<0) stop ("hcr3::argument oldssb must have a value greater than or equal to zero\n")
   if (ssbhcr<=0) stop ("hcr3::argument ssbhcr must have a value greater than zero\n")

   hcrmulty<-oldssb/ssbhcr

   return(c(hcrmulty,1))

}

############################################################ 
# HCR rule 4
# This function compares the simulated assessment estimate of (oldfbar) to a 
# fixed reference level fbarhcr. A value of hcrmult = fbarhcr/oldfbar is
# returned (resulting in an increase or reduction in the virtual fishery 
# depending on whether oldfbar < fbarhcr).

hcr4<-function(oldfbar,fbarhcr){

   #check for missing arguments
   if (missing(oldfbar) & missing(fbarhcr)) stop("hcr4::oldfbar and fbarhcr are mandatory arguments\n")
   if (missing(oldfbar)) stop("hcr4::oldfbar is a mandatory argument\n")
   if (missing(fbarhcr)) stop("hcr4::fbarhcr is a mandatory argument\n")

   #check arguments supplied are numeric
   if (!is.numeric(oldfbar)) stop ("hcr4::argument oldfbar must be numeric\n")
   if (!is.numeric(fbarhcr)) stop ("hcr4::argument fbarhcr must be numeric\n")
   if (oldfbar<0) stop ("hcr4::argument oldfbar must have a value greater than zero\n")
   if (fbarhcr<0) stop ("hcr4::argument fbarhcr must have a value greater than zero\n")

   #default value
   hcrmulty<-1

   if (oldfbar>fbarhcr) {
      hcrmulty<-fbarhcr/oldfbar
   }

   return(c(hcrmulty,1))

}

############################################################ 
# HCR rule 5
# if the new fbar exceeds that given in the fixed reference fbarhcr the
# fishery is closed

hcr5<-function(oldfbar,fbarhcr){

   #check for missing arguments
   if (missing(oldfbar) & missing(fbarhcr)) stop("hcr5::oldfbar and fbarhcr are mandatory arguments\n")
   if (missing(oldfbar)) stop("hcr5::oldfbar is a mandatory argument\n")
   if (missing(fbarhcr)) stop("hcr5::fbarhcr is a mandatory argument\n")

   #check arguments supplied are numeric
   if (!is.numeric(oldfbar)) stop ("hcr5::argument oldfbar must be numeric\n")
   if (!is.numeric(fbarhcr)) stop ("hcr5::argument fbarhcr must be numeric\n")
   if (oldfbar<0) stop ("hcr5::argument oldfbar must have a value greater than zero\n")
   if (fbarhcr<0) stop ("hcr5::argument fbarhcr must have a value greater than zero\n")

   #default value
   hcrmulty<-1

   if (oldfbar>fbarhcr) {
      hcrmulty<-0
   }

   return(c(hcrmulty,1))

}

############################################################ 
# HCR rule 6
# the multiplier is set to the ratio of the current fbar to the reference
# fbar, regardless of value

hcr6<-function(oldfbar,fbarhcr){

   #check for missing arguments
   if (missing(oldfbar) & missing(fbarhcr)) stop("hcr6::oldfbar and fbarhcr are mandatory arguments\n")
   if (missing(oldfbar)) stop("hcr6::oldfbar is a mandatory argument\n")
   if (missing(fbarhcr)) stop("hcr6::fbarhcr is a mandatory argument\n")

   #check arguments supplied are numeric
   if (!is.numeric(oldfbar)) stop ("hcr6::argument oldfbar must be numeric\n")
   if (!is.numeric(fbarhcr)) stop ("hcr6::argument fbarhcr must be numeric\n")
   if (oldfbar<1) stop ("hcr6::argument oldfbar must have a value greater than zero\n")
   if (fbarhcr<0) stop ("hcr6::argument fbarhcr must have a value greater than or equal to zero\n")

   hcrmulty<-fbarhcr/oldfbar

   return(c(hcrmulty,1))
  
}

###########################################################
# HCR rule 7
# different action taken depending in which range SSB sits
# SSB > Bpa            - tac set to constant catch value
# Blim <= SSB <= Bpa   - tac adjusted by SSB/Bpa
# SSB < Blim           - fishery is closed

hcr7<-function(SSB,Blim,Bpa){

   missing.args<-c()

   if (missing(SSB)) missing.args<-c(missing.args,"SSB")
   if (missing(Blim)) missing.args<-c(missing.args,"Blim")
   if (missing(Bpa)) missing.args<-c(missing.args,"Bpa")

   if (length(missing.args)>0) {
      stop(cat("hcr7::the following required argument(s) are missing:",missing.args,"\n"))
   }

   #check arguments supplied are numeric
   if (!is.numeric(SSB)) stop ("hcr7::argument SSB must be numeric\n")
   if (!is.numeric(Blim)) stop ("hcr7::argument Blim must be numeric\n")
   if (!is.numeric(Bpa)) stop ("hcr7::argument Bpa must be numeric\n")

   #Blim needs to be less than or equal to Bpa
   if (Blim > Bpa) stop ("hcr7::Blim must be less than or equal to Bpa")
  
   #all values should be greater than or equal to zero
   if (SSB < 0) stop ("hcr7::argument SSB must have a value greater than or equal to zero\n")
   if (Blim < 0) stop ("hcr7::argument Blim must have a value greater than or equal to zero\n")
   if (Bpa < 0) stop ("hcr7::argument Bpa must have a value greater than or equal to zero\n")

   if (SSB < Blim) {
      hcrmulty <- c(0,1)
   }

   if ((SSB >= Blim) & (SSB <= Bpa)) {
      if (Bpa > 0) {
	 hcrmulty <- c(SSB/Bpa,2)
      } else {
	 hcrmulty <- c(0,2)
      }
   }

   if(SSB > Bpa){
      hcrmulty <- 1 + ((SSB-Bpa)/Bpa)
      hcrmulty <- c(hcrmulty,3)
   }

   return(hcrmulty)

}

###########################################################
# USER DEFINED HCR rules

#HCR rule 8
#hcr8<-function() {

#   hcrmulty<-1

#   return(hcrmulty)

#}

