#histnormalplot.r
#Andrew Campbell
#Marine Institite, Galway
#andrew.campbell@marine.ie

#histogram & normal distribution plot
#plots a histogram of values of catch weight or ssb in a single year and imposes a normal distribution
#also displays stat. quantities of interest

histnormalplot<-function(runref = "missing",
			 res = "1",
			 year = "",
			 var = "1"){

   #runref - simulation reference
   #res - resolution reference
   #year - year to plot
   #var - variable (1=catch weight, 2=SSB)

   #store graphics settings
   def.par<-par(no.readonly=TRUE)

   #prompt for run reference
   if (missing(runref)) {
      runref <- winDialogString("Run Reference?","")
   }

   if (is.null(runref)) {stop("Cancelled\n")}

   if (nchar(runref)==0) {stop("Exit - invalid run reference")}

   #options file
   optionsfile = paste(FPRESS.Home(),"\\options\\","options",runref,".xml",sep="")

   #check options file exists
   if (!(file.exists(optionsfile))) {
      stop(paste("\nOptions file",optionsfile,"doesn't exist\nUnable to continue\n",sep=" "))
   }

   #load xml tree
   doc<-xmlTreeParse(optionsfile)

   #root xmlNode object
   root<-xmlRoot(doc)

   #read the relevant parameters for this run from the options file
   startyear <- as.integer(nodeval(root,"startyear",2000))
   years <- as.integer(nodeval(root,"years",1))
   resolution <- as.integer(nodeval(root,"resolution",1))
   ssbtest1 <- as.integer(nodeval(root,"ssbtest1",0))
   ssbtest2 <- as.integer(nodeval(root,"ssbtest2",0))

   #prompt for resolution reference, if simtype=1
   if (resolution > 0) {
      res <- winDialogString(paste("Resolution Reference (1-",resolution+1,")",sep=""),res)
      if (is.null(res)) {stop("Cancelled\n")}
   } else {
      res <- "1"
   }

   #prompt for year
   if (year > (startyear+years-1)) {
      year <- startyear+years-1
   }

   year <- winDialogString(paste("Year (",startyear,"-",startyear+years-1,")",sep=""),as.character(year))

   if (is.null(year)) {stop("Cancelled\n")}

   #prompt for variable
   var <- winDialogString("Variable - 1=catch weight, 2=SSB",var)

   if (is.null(var)) {stop("Cancelled\n")}

   #data file
   FLRFile = paste(FPRESS.Home(),"\\outdata\\FLRObjects_",runref,"_",res,".dat",sep="")

   #check FLR file exists
   if (!(file.exists(FLRFile))) {
      stop(paste("\nFLR data file",FLRFile,"doesn't exist\nUnable to continue\n",sep=" "))
   }

   #load data
   load(FLRFile)

   if (var=="1") {
      Data<-quantSums(CatchWeight)
      xlabel<-"Catch Weight"
   } else if (var=="2") {
      Data<-quantSums(NewSpawnMass)
      xlabel<-"SSB"
   } else {
      stop("Invalid choice")
   }
  
   YearData<-as.vector(Data[,year,,,])
   mn<-mean(YearData)
   sd<-sd(YearData)
   med=median(YearData)
   xmax<-2*max(YearData)
   title<-paste("Run ",runref,"_",res,",",year,sep="")

   h<-hist(YearData,breaks=50,xlim=c(0,xmax),xlab=xlabel,main=title)

   maxval<-normaldist(mean(YearData),sd(YearData),mean(YearData))

   scaling<-max(h$counts)/maxval

   curve(scaling*normaldist(x,sd(YearData),mean(YearData)),0,xmax,add=TRUE)

   lines(c(mn,mn),c(-10,max(h$counts)),lty=2)
   lines(c(mn-1.96*sd,mn-1.96*sd),c(-10,scaling*normaldist(mn-1.96*sd,sd,mn)),lty=2)
   lines(c(mn-2.58*sd,mn-2.58*sd),c(-10,scaling*normaldist(mn-2.58*sd,sd,mn)),lty=2)
   lines(c(mn+1.96*sd,mn+1.96*sd),c(-10,scaling*normaldist(mn+1.96*sd,sd,mn)),lty=2)
   lines(c(mn+2.58*sd,mn+2.58*sd),c(-10,scaling*normaldist(mn+2.58*sd,sd,mn)),lty=2)

   #if this is a plot of SSB draw lines for ssbtest1 and ssbtest2
   if (var=="2") {
      lines(c(ssbtest1,ssbtest1),c(0,-10),lty=1,lwd=2)
      lines(c(ssbtest2,ssbtest2),c(0,-10),lty=1,lwd=2)
   }
   
   text(0,max(h$counts),paste("mean:",format(mn,digits=6)),pos=4,cex=0.6)
   text(0,0.975*max(h$counts),paste("median:",format(med,digits=6)),pos=4,cex=0.6)
   text(0,0.95*max(h$counts),paste("sd:",format(sd,digits=6)),pos=4,cex=0.6)
   text(0,0.925*max(h$counts),paste("95% limits:",format(mn-1.96*sd,digits=6),"-",format(mn+1.96*sd,digits=6)),pos=4,cex=0.6)
   text(0,0.9*max(h$counts),paste("99% limits:",format(mn-2.58*sd,digits=6),"-",format(mn+2.58*sd,digits=6)),pos=4,cex=0.6)

   #restore graphics settings
   par(def.par)
}

