#recrplot.r
#Andrew Campbell
#Marine Institite, Galway
#andrew.campbell@marine.ie

#recruitment plot 
#plots of recruitment versus year
#for simulations with more than 1 resolution 
#i.e. varying ftac, user can select single, all or a contiguous range
#of resolution (ftac) values to plot

recrplot<-function(runref = "missing",
		   box.whisker = 1,
		   conf.int = 1,
		   yminval = "missing",
		   ymaxval = "missing"){

   #runref - simulation reference
   #box.whisker = box & whisker plots?
   #conf.int = plot confidence intervals?
   #yminval - minimum y value
   #ymaxval = maximum y value

   #prompt for run reference
   if (missing(runref)) { 
      runref<-winDialogString("Run Reference?","")
   }

   if (is.null(runref)) {stop("Cancelled\n")}

   if (nchar(runref)==0) {stop("Exit - invalid run reference")}

   #construct options filename
   optionsfile = paste(FPRESS.Home(),"\\options\\options",runref,".xml",sep="")

   #check options file exists
   if (!(file.exists(optionsfile))) {
      stop(paste("\nOptions file",optionsfile,"doesn't exist\nUnable to continue\n",sep=" "))
   }

   #load xml tree
   doc<-xmlTreeParse(optionsfile)

   #root doc
   root<-xmlRoot(doc)

   #node values
   resolution <- as.integer(nodeval(root,"resolution",1))
   iterations <- as.integer(nodeval(root,"iterations",1))
   startyear <- as.integer(nodeval(root,"startyear",1))
   years <- as.integer(nodeval(root,"years",1))
   varmin <- as.numeric(nodeval(root,"varmin",1))
   varmax <- as.numeric(nodeval(root,"varmax",1))
   variable <- as.integer(nodeval(root,"variable",1))
   minage <- as.integer(nodeval(root,"minage",1))
   maxage <- as.integer(nodeval(root,"maxage",1))

   #number of age classes
   numages <- maxage - minage + 1

   #last year
   endyear <- startyear + years - 1

   #default value for the number of plots
   plots<-1
   min.res<-1
   max.res<-1

   #if resolution for this simulation is greater than 0 prompt for the resolutions to output
   if (resolution>0) {

      if (variable==1) {
	 var <- "F"
      } else {
	 var <- "TAC"
      }

      resvals<-winDialogString(paste(as.character(resolution+1)," ",var," values ","(",varmin,"-",varmax,").\n",
				     "Enter first, last simulation to plot (* for all)",sep=""),"")

      if (is.null(resvals)) {stop("Cancelled\n")}

      if (nchar(resvals)==0) {stop("Invalid Resolution")}

      res.limits<-unlist(strsplit(resvals,",",fixed=TRUE))

      if (length(res.limits)==1) {
         if (res.limits[1]=="*") {
	    min.res<-1
	    max_.res<-resolution
	    total.plots<-resolution+1
	 } else {
	    min.res<-as.integer(res.limits[1])
	    max.res<-as.integer(res.limits[1])
	    total.plots<-1
	 }
      } else if (length(res.limits)==2) {
	 min.res<-as.integer(res.limits[1])
	 max.res<-as.integer(res.limits[2])
	 total.plots<-max.res-min.res+1
      } else {
	 stop("Invalid resolution\n")
      }

   } else {

      total.plots <- 1

   }

   #how many rows of plots?
   rows<-ceiling(sqrt(total.plots))

   #scaling factor for text on plots (cex argument) depends on the number of rows of plots
   if (rows>5) {
      txtscale <- 0.6
   } else if (rows>4) {
      txtscale <- 0.7 
   } else if (rows>3) {
      txtscale <- 0.8 
   } else if (rows>2) {
      txtscale <- 0.9 
   } else {
      txtscale <- 1.0
   }

   data<-seq(1:(rows*rows))

   #outer margins
   par.omi<-par("omi")
   par(omi=c(0.2,0.2,0,0))

   #individual plot margin lines
   par.mar<-par("mar")
   par(mar=c(2,2,1,0))

   #set up plot area with the required number of plots
   layout(matrix(data, rows, rows, byrow = TRUE))

   #vector of years
   year<-seq(startyear,length=years)

   #box width for box and whisker plots
   box.width<-(max(year)-min(year))/(4*length(year))

   #recr data array
   recarray <- array(0,
		     dim = c(1,years,1,1,iterations,total.plots),
		     dimnames = list(age='all',year=startyear:endyear,unit='unique',season='all',area=1:iterations,res=1:total.plots))

   #loop through simulations, reading in data
   #this is done prior to any plotting to determine common limits for multiple plots
   for(plot.counter in 0:(total.plots-1))
   {

      res<-min.res+plot.counter

      #data file
      FLRFile = paste(FPRESS.Home(),"\\outdata\\FLRObjects_",runref,"_",res,".dat",sep="")

      #check FLR file exists
      if (!(file.exists(FLRFile))) {
	 stop(paste("\nFLR data file",FLRFile,"doesn't exist\nUnable to continue\n",sep=" "))
      }

      #load FLQuant objects
      load(FLRFile)

      #update local array
      recarray[,,,,,plot.counter+1] <- Recruits
   }

   minrec <- min(recarray)
   maxrec <- max(recarray)
		     
   #loop thro' plots
   for(plot.counter in 0:(total.plots-1))
   {

      #current plot resolution number
      res<-min.res+plot.counter

      #default title
      title<-"Recruitment v Year"

      if (resolution>0) {

	 #calculate the appropriate ftac value for the current resolution
         ftac<-max(((varmax-varmin)*(res-1)/resolution)+varmin,0.0001)

	 if (variable==1) {
	    title<-paste("f = ",round(ftac,digits=4))
	 } else {
	    title<-paste("tac = ",round(ftac,digits=0))
	 }

      }

      #extract data from recarray, cast to an FLQuant and plot
      recdata<-FLQuant(array(recarray[,,,,,plot.counter+1],
			     dim=c(1,years,1,1,iterations),
			     dimnames=list(age='all',year=startyear:endyear,unit='unique',season='all',area=1:iterations)))

      #calc quantities of interest
      recmean<-areaMeans(recdata)                               #mean (over iterations)
      recmedian<-apply(recdata,2,median)
      recres<-sweep(recdata,2,recmean,"-")                      #residuals
      recstddev<-sqrt(areaSums(recres*recres)/(iterations-1))   #standard deviation
      reccv<-recstddev/recmean                                  #coefficient of variation

      if (missing(ymaxval)) {
	 ymax <- maxrec
	 #if (max(recres,na.rm=FALSE)>0) {
	 #   ymax<-1.5*max(recmean+1.96*recmean*reccv)
	 #} else {
	 #   ymax=max(recmean,na.rm=TRUE)
	 #}
      } else {
	 ymax <- ymaxval
      }

      if (missing(yminval)) {
	 ymin <- minrec
	 #ymin<-0
      } else {
	 ymin <- yminval
      }

      rdum<-seq(ymin,ymax,length=length(year))    

      #create plot
      plot(year,rdum,axes=F,type="n")
     
      #median
      lines(year,recmedian,lwd=2)

      #95/99% confidence limits
      if(conf.int==1){
	 lines(year,recmean-1.96*reccv*recmean,lty=2)
	 lines(year,recmean+1.96*reccv*recmean,lty=2)
	 lines(year,recmean-2.58*reccv*recmean,lty=2)
	 lines(year,recmean+2.58*reccv*recmean,lty=2)
      }

      #box whisker plots
      if(box.whisker==1){
	 for(i in 1:years){
	    recvals<-as.numeric(Recruits[,i,,,])
	    boxplot(recvals,range=0.1,boxwex=box.width,add=TRUE,at=i+startyear-1,axes=F)
	 }
      }

      #write title as a margin text string
      #mtext(title,side=3,line=0,cex=0.65)
      text(x=min(year),y=ymax,title,pos=4,cex=txtscale)

      #bottom row graphs get xaxis labels and text
      #number of columns is the same as number of rows so that
      #if plot.counter is >= total.plots-rows that plot is the bottom plot for that column
      if ((resolution==0) | (plot.counter>=(total.plots-rows))) {
	 axis(side=1)
	 #mtext("Year",side=1,line=2,cex=0.65)
	 mtext("Year",side=1,line=2,cex=txtscale)
      } else {
	 axis(side=1,labels=FALSE)
      }

      #LHS graphs get y-axis labels and text
      if ((plot.counter%%rows)==0) {
	 axis(side=2)
	 #mtext("Recruits",side=2,line=2,cex=0.65)
	 mtext("Recruits",side=2,line=2,cex=txtscale)
      } else {
	 axis(side=2,labels=FALSE)
      }

   }

   #restore plotting defaults
   par(omi=par.omi)
   par(mar=par.mar)

}
