#recruit.r
#Andrew Campbell
#Marine Institite, Galway
#andrew.campbell@marine.ie
#10/07/2006

#Recruitment functions

############################################################
#Recruitment function 0 (fixed value)

#This function returns a value for recruits given by the mean recruitment 
#value specified by the recmean parameter

recruit0<-function(recmean){

   #recmean is a required parameter that must be a value greater 
   #than or equal to zero
   if (missing(recmean)) stop("recruit0::recmean is a mandatory parameter\n")
   if (!is.numeric(recmean)) stop ("recruit0::argument recmean must be numeric\n")
   if (recmean<0) stop ("recruit0::argument recmean must be a value greater than or equal to zero\n")

   recruits<-recmean

   return(recruits)

}

############################################################
#Recruitment function 1 (random from normal):

#This function initially sets the recruits parameter to be negative and 
#continues to select values for recruits from a Normal distribution until
#a non-negative value is chosen and returned (i.e. a truncated Normal 
#distribution is used). The Normal distribution used has mean = recmean and 
#standard deviation = reccv*recmean, where the parameters recmean and reccv 
#are those specified at the start of the run

recruit1<-function(recmean,reccv){

   #recmean is a required parameter that must be a numeric value 
   #greater than or equal to zero
   #reccv is a required parameter that must be a numeric value 
   #greater than or equal to zero

   if ((missing(recmean) & missing(reccv))) stop("recruit1::recmean and reccv are mandatory parameters\n")
   if (missing(recmean)) stop("recruit1::recmean is a mandatory parameter\n")
   if (missing(reccv)) stop("recruit1::reccv is a mandatory parameter\n")

   if (!is.numeric(recmean)) stop ("recruit1::argument recmean must be numeric\n")
   if (recmean<0) stop ("recruit1::argument recmean must be a value greater than or equal to zero\n")
   if (!is.numeric(reccv)) stop ("recruit1::argument reccv must be numeric\n")
   if (reccv<0) stop ("recruit1::argument reccv must be a value greater than or equal to zero\n")

   #initial value
   recruits<--1

   while (recruits<0){
      recruits <- rnorm(1,recmean,reccv*recmean)
   }

   return(recruits)

}

############################################################
#Recruitment function 2 (random from previous years):

#This function uses the spwnfrec vector, which stores the historical recruit 
#values (up to the year datarecyear) from the recruits.txt file. The function
#sets up a vector of numbers 1 to k, where k is the length of the spwnfrec vector, 
#and randomly chooses one of these numbers from the vector k. The entry in the 
#spwnfrec vector corresponding to this random number is then chosen and this 
#historical value is returned for the recruits parameter (e.g. if the random number 
#drawn is 7, then the 7th entry in the spwnfrec vector is chosen as recruits). The 
#end result is that a value is randomly drawn from the spwnfrec vector, and all 
#historical values have the same probability of being drawn

recruit2<-function(spwnfnyr,spwnfnrec){

   #spwnfnyr and spwnfnrec are both required arguments
   #spwnfnyear must be a vector of length 1 or more of positive values
   #spwnfnrec must be a vector of length 1 or more of real positive values
   #spwnfnyear and spwnfnrec must both be of the same length

   if ((missing(spwnfnyr) & missing(spwnfnrec))) stop("recruit2::spwnfnyr and spwnfnrec are mandatory parameters\n")
   if (missing(spwnfnyr)) stop("recruit2::spwnfnyr is a mandatory parameter\n")
   if (missing(spwnfnrec)) stop("recruit2::spwnfnrec is a mandatory parameter\n")

   if (length(spwnfnyr)<1) stop("recruit2::argument spwnfnyr must be a vector of length >= 1\n")
   if (length(spwnfnrec)<1) stop("recruit2::argument spwnfnrec must be a vector of length >= 1\n")

   if (!is.numeric(spwnfnyr)) stop ("recruit2::argument spwnfnyr must be of numeric type\n")
   if (!is.numeric(spwnfnrec)) stop ("recruit2::argument spwnfnrec must be of numeric type\n")

   if (!(length(spwnfnyr) == length(spwnfnrec))) stop ("recruit2::arguments spwnfnyr and 
   spwnfnrec must be vectors of the same length\n")

   if (sum(spwnfnyr<0)>0) stop ("recruit2::all spwnfnyr values must be greater than zero\n")
   if (sum(spwnfnrec<0)>0) stop ("recruit2::all spwnfnrec values must be greater than zero\n")

   n<-length(spwnfnrec)
   k<-1:n

   rn<-sample(k,1)
   yearn<-spwnfnyr[rn]
   recruits<-spwnfnrec[rn]

   return(recruits)

}

############################################################
#Recruitment function 3 (random from previous years with two groupings for ssb):

#As recruitment function 2, except that the parameter spwnfnssb corresponding to 
#the column of historical SSB values (up to datarecyear) from the recruits.txt file is used. 
#If the virtual SSB is above ssbcut then only historical recruitment with corresponding 
#historical SSB above this ssbcut level will be drawn and returned as recruits. 
#A similar process applies if the virtual SSB is below ssbcut. 

recruit3<-function(ssb,spwnfnyr,spwnfnrec,spwnfnssb,ssbcut){

   #all arguments are required
   #all arguments must be numeric
   #spwnfnyear, spwnfnrec & spwnfnssb must be vectors of length >= 1
   #spwnfnyear, spwnfnrec & spwnfnssb must all be of the same length
   #spwnfnssb needs contain values that are below ssbcut if ssb < ssbcut
   #spwnfnssb needs to contain values that are above ssbcut if ssb >= ssbcut

   missing.args<-c()

   if (missing(ssb)) missing.args<-c(missing.args,"ssb")
   if (missing(spwnfnyr)) missing.args<-c(missing.args,"spwnfnyr")
   if (missing(spwnfnrec)) missing.args<-c(missing.args,"spwnfnrec")
   if (missing(spwnfnssb)) missing.args<-c(missing.args,"spwnfnssb")
   if (missing(ssbcut)) missing.args<-c(missing.args,"ssbcut")

   if (length(missing.args)>0) {
      stop(cat("recruit3::the following required argument(s) are missing:",missing.args,"\n"))
   }

   if (length(spwnfnyr)<1) stop("recruit3::argument spwnfnyr must be a vector of length >= 1\n")
   if (length(spwnfnrec)<1) stop("recruit3::argument spwnfnrec must be a vector of length >= 1\n")
   if (length(spwnfnssb)<1) stop("recruit3::argument spwnfnssb must be a vector of length >= 1\n")

   if (!is.numeric(ssb)) stop ("recruit3::argument ssb must be of numeric type\n")
   if (!is.numeric(spwnfnyr)) stop ("recruit3::argument spwnfnyr must be of numeric type\n")
   if (!is.numeric(spwnfnrec)) stop ("recruit3::argument spwnfnrec must be of numeric type\n")
   if (!is.numeric(spwnfnssb)) stop ("recruit3::argument spwnfnssb must be of numeric type\n")
   if (!is.numeric(ssbcut)) stop ("recruit3::argument ssbcut must be of numeric type\n")

   if (!(length(spwnfnyr) == length(spwnfnrec))) stop ("recruit3::arguments spwnfnyr and 
   spwnfnrec must be vectors of the same length\n")
   if (!(length(spwnfnyr) == length(spwnfnssb))) stop ("recruit3::arguments spwnfnyr and 
   spwnfnssb must be vectors of the same length\n")

   if (sum(spwnfnyr<0)>0) stop ("recruit3::all spwnfnyr values must be greater than zero\n")
   if (sum(spwnfnrec<0)>0) stop ("recruit3::all spwnfnrec values must be greater than zero\n")
   if (sum(spwnfnssb<0)>0) stop ("recruit3::all spwnfnssb values must be greater than zero\n")

   if ((ssb<ssbcut) & (sum(spwnfnssb<ssbcut)==0)) stop ("recruit3::if ssb<ssbcut the vector 
   spwnfnssb needs to contain at least one value lower than ssbcut\n")
   
   if ((ssb>=ssbcut) & (sum(spwnfnssb>=ssbcut)==0)) stop ("recruit3::if ssb>=ssbcut the vector 
   spwnfnssb needs to contain at least one value >= ssbcut\n")

   n<-length(spwnfnrec)
   k<-1:n
   recruits<-0

   if (ssb < ssbcut) {

      ssbn<-2*ssbcut
    
      while (ssbn > ssbcut) {

	 rn<-sample(k,1)
	 yearn<-spwnfnyr[rn]
	 ssbn<-spwnfnssb[rn]
	 recruits<-spwnfnrec[rn]}
  
  } else {

      ssbn<-0.5*ssbcut
    
      while (ssbn < ssbcut) {

	 rn<-sample(k,1)
	 yearn<-spwnfnyr[rn]
	 ssbn<-spwnfnssb[rn]
	 recruits<-spwnfnrec[rn]
      }
  
  }
  
  return(recruits)
  
}

############################################################
#Recruitment function 4 (Ricker map fitted to data from previous years):

#This function fits a Ricker model to the historical SSB-recruitment 
#data in the recruits.txt file (up to datarecyear). As Ricker is an exponential 
#model, a linear fit (lm command in R) is made to the log of the data. The 
#Ricker model parameters are then calculated from the output of the linear fit, 
#and using the virtual SSB level, a value of recruits is calculated and returned.

recruit4<-function(ssb,spwnfnrec,spwnfnssb){

   #all arguments are required
   #all arguments must be numeric
   #ssb must be >= 0
   #spwnfnrec and spenfnssb must be > 0
   #spwnfnrec and spwnfnssb must be of the same length

   missing.args<-c()

   if (missing(ssb)) missing.args<-c(missing.args,"ssb")
   if (missing(spwnfnrec)) missing.args<-c(missing.args,"spwnfnrec")
   if (missing(spwnfnssb)) missing.args<-c(missing.args,"spwnfnssb")

   if (length(missing.args)>0) {
      stop(cat("recruit4::the following required argument(s) are missing:",missing.args,"\n"))
   }

   if (length(spwnfnyr)<1) stop("recruit3::argument spwnfnyr must be a vector of length >= 1\n")
   if (length(spwnfnrec)<1) stop("recruit4::argument spwnfnyr must be a vector of length >= 1\n")
   if (length(spwnfnssb)<1) stop("recruit4::argument spwnfnrec must be a vector of length >= 1\n")

   if (!is.numeric(ssb)) stop ("recruit4::argument ssb must be of numeric type\n")
   if (!is.numeric(spwnfnrec)) stop ("recruit4::argument spwnfnrec must be of numeric type\n")
   if (!is.numeric(spwnfnssb)) stop ("recruit4::argument spwnfnssb must be of numeric type\n")

   if (ssb<0) stop ("recruit4::argument ssb must be > 0\n")
   if (sum(spwnfnrec<0)>0) stop ("recruit4::all spwnfnrec values must be greater than zero\n")
   if (sum(spwnfnssb<0)>0) stop ("recruit4::all spwnfnssb values must be greater than zero\n")

   if (!(length(spwnfnrec == length(spwnfnssb)))) stop ("recruit4::arguments spwnfnrec and 
   spwnfnssb must be vectors of the same length\n")

   y <- log(spwnfnrec/spwnfnssb)	 
   x <- spwnfnssb
   z <- coef(lm(y ~ x))
   alpha <- exp(z[1])
   beta <- z[2]

   recruits<-ssb*alpha*exp(beta*ssb)

   return(recruits)


}

############################################################ 
#Recruitment function 5 (Ricker map fitted to data from previous years with added noise):

#As recruitment function 4 except noise is added to the Ricker value of recruits. 
#A CV value (reccv) is calculated from the historical data (up to datarecyear). 
#The calculated value for recruitment from the Ricker model is then used as the mean 
#value of a Normal distribution with a standard deviation calculated from reccv. A 
#non-negative value for recruits is then randomly drawn from this Normal distribution 
#and returned. 

recruit5<-function(ssb,spwnfnrec,spwnfnssb){

   #all arguments are required
   #all arguments must be numeric
   #ssb must be >= 0
   #spwnfnrec and spwnfnssb must be > 0
   #spwnfnrec and spwnfnssb must be of the same length

   missing.args<-c()

   if (missing(ssb)) missing.args<-c(missing.args,"ssb")
   if (missing(spwnfnrec)) missing.args<-c(missing.args,"spwnfnrec")
   if (missing(spwnfnssb)) missing.args<-c(missing.args,"spwnfnssb")

   if (length(missing.args)>0) {
      stop(cat("recruit5::the following required argument(s) are missing:",missing.args,"\n"))
   }

   if (length(spwnfnrec)<1) stop("recruit5::argument spwnfnyr must be a vector of length >= 1\n")
   if (length(spwnfnssb)<1) stop("recruit5::argument spwnfnrec must be a vector of length >= 1\n")

   if (!is.numeric(ssb)) stop ("recruit5::argument ssb must be of numeric type\n")
   if (!is.numeric(spwnfnrec)) stop ("recruit5::argument spwnfnrec must be of numeric type\n")
   if (!is.numeric(spwnfnssb)) stop ("recruit5::argument spwnfnssb must be of numeric type\n")

   if (ssb<0) stop ("recruit5::argument ssb must be > 0\n")
   if (sum(spwnfnrec<0)>0) stop ("recruit5::all spwnfnrec values must be greater than zero\n")
   if (sum(spwnfnssb<0)>0) stop ("recruit5::all spwnfnssb values must be greater than zero\n")

   if (!(length(spwnfnrec == length(spwnfnssb)))) stop ("recruit5::arguments spwnfnrec and 
   spwnfnssb must be vectors of the same length\n")

   y <- log(spwnfnrec/spwnfnssb)	 
   x<-spwnfnssb

   lsq<-lm(y~x)
   z<-coef(lsq)

   junk<-summary(lsq)

   #shouldn't really be calculating this every time
   reccv<-sqrt(var(spwnfnrec,y=NULL,na.rm=FALSE))/mean(spwnfnrec) #cv from absolute recruit values
  
   alpha<-exp(z[1])
   beta<-z[2]

   recruitmn<-ssb*alpha*exp(beta*ssb)

   recruits<--1
  
   while (recruits<0) {
      recruits<-rnorm(1,recruitmn,reccv*recruitmn)
   }

   return(recruits)

}

############################################################
#Recruitment function 6 (Segmented regression - data already in function):

#This function uses the ssbcut, recmean and seggrad parameter values 
#to calculate a 'hockey-stick' stock-recruit relation. In principle, it should 
#be possible to calculate these parameter values from the historical data in the 
#recruits.txt file but the program does not yet do this. If the virtual SSB is 
#greater than the specified SSB level, ssbcut, then recruitment is just taken to be 
#the value given by recmean. If the virtual SSB < ssbcut, then the recruitment is 
#calculated as a linear relation to SSB (from zero), where the gradient of the linear 
#function is given by seggrad. Note that to be correct the following relation must 
#hold: ssbcut*seggrad = recmean. The calculated recruitment value is then returned as 
#recruits.

recruit6 <- function(ssb,ssbcut,recmean,seggrad) {

   #all parameters are required
   #all should be numeric and positive

   missing.args<-c()

   if (missing(ssb)) missing.args<-c(missing.args,"ssb")
   if (missing(ssbcut)) missing.args<-c(missing.args,"ssbcut")
   if (missing(recmean)) missing.args<-c(missing.args,"recmean")
   if (missing(seggrad)) missing.args<-c(missing.args,"seggrad")

   if (length(missing.args)>0) {
      stop(cat("recruit6::the following required argument(s) are missing:",missing.args,"\n"))
   }

   if (!is.numeric(ssb)) stop ("recruit6::argument ssb must be of numeric type\n")
   if (!is.numeric(ssbcut)) stop ("recruit6::argument ssbcut must be of numeric type\n")
   if (!is.numeric(recmean)) stop ("recruit6::argument recmean must be of numeric type\n")
   if (!is.numeric(seggrad)) stop ("recruit6::argument seggrad must be of numeric type\n")

   if (ssb<0) stop ("recruit6::argument ssb must be >= 0\n")
   if (ssbcut<0) stop ("recruit6::argument ssbcut must be >= 0\n")
   if (recmean<0) stop ("recruit6::argument recmean must be >= 0\n")
   if (seggrad<0) stop ("recruit6::argument seggrad must be >= 0\n")

   if (ssb==0) {

      recruits<-0

   } else {

      if (ssb > ssbcut) {
	 recmean1 <- recmean
      } else {
	 recmean1 <- ssb*seggrad
      }
   }
   
   recruits<-recmean1

   return(recruits)

}

############################################################
#Recruitment function 7 (Segmented regression with added noise - data already in function):

#As recruitment function 6, except noise is added to the recruitment value. The 
#recruitment value from the 'hockey-stick' function is used as the mean value of a 
#Normal distribution with standard deviation calculated from this mean value and the 
#specified value of reccv from the FPRESSopt.r file. A non-negative value is then 
#drawn from this Normal distribution and returned as recruits.

recruit7 <- function(ssb,ssbcut,recmean,seggrad,reccv) {

   missing.args<-c()

   if (missing(ssb)) missing.args<-c(missing.args,"ssb")
   if (missing(ssbcut)) missing.args<-c(missing.args,"ssbcut")
   if (missing(recmean)) missing.args<-c(missing.args,"recmean")
   if (missing(seggrad)) missing.args<-c(missing.args,"seggrad")
   if (missing(reccv)) missing.args<-c(missing.args,"reccv")

   if (length(missing.args)>0) {
      stop(cat("recruity::the following required argument(s) are missing:",missing.args,"\n"))
   }

   if (!is.numeric(ssb)) stop ("recruit7::argument ssb must be of numeric type\n")
   if (!is.numeric(ssbcut)) stop ("recruit7::argument ssbcut must be of numeric type\n")
   if (!is.numeric(recmean)) stop ("recruit7::argument recmean must be of numeric type\n")
   if (!is.numeric(seggrad)) stop ("recruit7::argument seggrad must be of numeric type\n")
   if (!is.numeric(reccv)) stop ("recruit7::argument reccv must be of numeric type\n")

   if (ssb<0) stop ("recruit7::argument ssb must be >= 0\n")
   if (ssbcut<0) stop ("recruit7::argument ssbcut must be >= 0\n")
   if (recmean<0) stop ("recruit7::argument recmean must be >= 0\n")
   if (seggrad<0) stop ("recruit7::argument seggrad must be >= 0\n")
   if (reccv<0) stop ("recruit7::argument reccv must be >= 0\n")    # EINAR replaced reccvd with reccv

   if (ssb==0) {
   
      recruits <- 0
      
   } else {

      if (ssb>ssbcut) {
	 
	 recmean <- recmean
	 
      } else {
      
	 recmean <- ssb*seggrad
	 
      }

      rectst<--recmean
   
      while (rectst<0) {

	 rectst<-rnorm(1,recmean,recmean*reccv)

}
   
      recruits<-rectst
  
   }

   return(recruits)

}

############################################################
#user defined recruitment functions

#recruit8<-function(){

#   recruits <- 0

#}
