#rickerplot.r
#Andrew Campbell
#Marine Institite, Galway
#andrew.campbell@marine.ie

####################################################################
# Plot of simulated and Ricker recruit values.######################
####################################################################

rickerplot <- function(runref = "missing",
		       res = "missing",
		       legend = 1,
		       legcoordx = 0,
		       legcoordy = 1){

   #runref - simulation reference
   #legend - plot legend?
   #legcoordx - legend x-coordinate
   #legcoordy - legend y-coordinate

   #save graphics settings
   def.par<-par(no.readonly=TRUE)

   #prompt for run reference
   if (missing(runref)) {
      runref <- winDialogString("Run Reference?","")
   }

   if (is.null(runref)) {stop("Cancelled\n")}

   if (nchar(runref)==0) {stop("Exit - invalid run reference")}

   #options file
   optionsfile = paste(FPRESS.Home(),"\\options\\","options",runref,".xml",sep="")

   #check options file exists
   if (!(file.exists(optionsfile))) {
      stop(paste("\nOptions file",optionsfile,"doesn't exist\nUnable to continue\n",sep=" "))
   }

   #load xml tree
   doc<-xmlTreeParse(optionsfile)

   #root xmlNode object
   root<-xmlRoot(doc)

   #node values
   recfile <- nodeval(root,"recfile","")
   stockname <- nodeval(root,"stockname","")
   minage <- as.integer(nodeval(root,"minage",0))
   rec <- as.integer(nodeval(root,"rec",0))
   iterations <- as.integer(nodeval(root,"iterations",0))
   resolution <- as.integer(nodeval(root,"resolution",0))

   #check if the supplied recruits file exists
   if (!file.exists(recfile)) {stop(paste("Recruits file",recfile,"does not exist\n",sep=" "))}
      
   #abort the plot if the simulation did not employ a Ricker recruitment model
   if (rec<4 || rec>5) {cat("WARNING - This simulation did not use a Ricker model for recruitment\n")}
 
   #prompt for resolution reference, if multiple FTAC values used
   if (missing(res)) {
      if (resolution > 0) {
         res <- winDialogString(paste("Resolution Reference (1-",resolution+1,")",sep=""),"")

	 if (is.null(res)) {stop("Cancelled\n")}

      } else {
         res <- "1"
      }
   }

   #construct data filename
   FLRFile = paste(FPRESS.Home(),"\\outdata\\","FLRObjects_",runref,"_",res,".dat",sep="")

   #and check it exists
   if (!file.exists(recfile)) {stop(paste("FLR file",FLRFile,"does not exist\n",sep=" "))}
  
   #load FLQuant objects
   load(FLRFile)

   input <- scan(recfile,list(0,0,0),quiet=TRUE) 
   spwnfnyr <- input[[1]]   #recruitment data year vector
   spwnfnrec <- input[[2]]  #recruitment data recruitment numbers vector
   spwnfnssb <- input[[3]]  #recruitment data ssb vector

   layout(matrix(c(1), 1, 1, byrow = TRUE))

   #ricker curve parameters
   y <- log(spwnfnrec/spwnfnssb)
   x <- spwnfnssb
   z <- coef(lm(y ~ x))
   alpha <- exp(z[1])
   beta <- z[2]

   #recsim <- as.numeric(areaMeans(Recruits))
   #ssbm <- as.numeric(areaMeans(quantSums(NewSpawnMass)))

   recsim <- as.numeric(Recruits)
   ssbm <- as.numeric(quantSums(NewSpawnMass))
   
   l <- length(ssbm)
   ssbn <- 1.2*max(c(ssbm,spwnfnssb))
   recn <- 1.2*max(c(recsim,spwnfnrec))
   ssbdum <- seq(0,ssbn,length=500)
   recdum <- ssbdum*alpha*exp(beta*ssbdum)

   plot(c(0,ssbn),c(0,recn),xlab="SSB (tonnes)",ylab="Recruits (000s)",main=paste("SSB v Recruits for ",stockname),type="n")
   #ricker plot
   lines(ssbdum,recdum)
   #historical values
   points(spwnfnssb,spwnfnrec,pch="+",cex=0.5,col="black")
   #simulation
   points(ssbm,recsim,pch="o",cex=0.5,col="red")

   if(legend==1){
      leglabs<-c(paste("Observed recruits (",min(spwnfnyr)+minage,"-",max(spwnfnyr)+minage,")"), "Simulated recruits", "Ricker function")
      legvals<-c("+","o","-")
      legcols<-c("black","black","black")
      legend(ssbn*legcoordx,recn*legcoordy,leglabs,pch=legvals,col=legcols)
   }

   newyears<-spwnfnyr+minage

   for(i in 1:length(spwnfnssb)){
      text(spwnfnssb[i],spwnfnrec[i]+(recn/80),labels=newyears[i],cex=0.5)
   }
   #restore graphics settings
   par(def.par)
}
