#FPRESS.run.r     Additional changes by Einar - search for EINAR
#Andrew Campbell
#Marine Institite, Galway
#andrew.campbell@marine.ie
#V1 10/07/2006
#V2 28/08/2006	  completed comments, some minor modifications to make code more readable
#V2 10/10/2006	  tacopt,simtype,tacmin,tacmax,tacbias,taccv,fmin,fmax,fbarcv,fbias replaced with variable,resolution,varmin,varmax,varcv,varbias
#V2 16/10/2006	  function changed to FPRESS.Run(), runref missing, output directory changed to outdata

FPRESS.Run<-function(runref="missing",batch=0,debug=0)
{
   #runref - the simulation reference. Function will examine file options<runref>.xml
   #for model parameters
   #batch - batch flag. If true(1), the function will not prompt user for any settings 
   #and will use defaults.
   #debug flag - if true(1), print debug information to the screen

   if (!(is.numeric(batch))) stop("FPRESS.Run::if batch argument is supplied it must assume a value of 0 or 1\n")
   if (!(is.numeric(debug))) stop("FPRESS.Run::if debug argument is supplied it must assume a value of 0 or 1\n")
   if (!(batch==1) & !(batch==0)) stop("FPRESS.Run::if batch argument is supplied it must assume a value of 0 or 1\n")
   if (!(debug==1) & !(debug==0)) stop("FPRESS.Run::if debug argument is supplied it must assume a value of 0 or 1\n")

   #save the current working directory
   currwd<-getwd()

   #get the FPRESS home directory
   home <- FPRESS.Home()

   if (debug==1) {cat("home=",home,"\n")}

   #check that the environment variable FPRESSHome is not empty
   if (nchar(home)==0) stop("Exiting - no FPRESSHome environment variable")

   #prompt for the run reference (only if not in batch mode)
   if (batch==0) {
      if (missing(runref)) { 
	 runref<-winDialogString("Run Reference?","")
      }
   } else {
      if (missing(runref)) stop("No run reference")
   }

   if (nchar(runref)==0) stop("Exiting - no run reference provided")

   #create an empty vector into which the names of the objects to be saved will be written
   #savedobjs<-c()     Redundant, nothing is really done with this - see line 299 and onward

   #construct the name of the relevant xml file containing simulation parameters
   #the file is located in the folder given in variable options.path which is set
   #in the header script (header.r). <runref> is appended to "options" to construct
   #the filename
   optionsfile = paste(FPRESS.Home(),"\\options\\","options",runref,".xml",sep="")

   #check options file exists
   if (!(file.exists(optionsfile))) {
      stop(paste("\nOptions file",optionsfile,"doesn't exist\nUnable to continue\n",sep=" "))
   }

   #load the simulation options xml document into memory using the xml library xmlTreeParse function
   doc<-xmlTreeParse(optionsfile)

   #root xmlNode object
   root<-xmlRoot(doc)
 
   #read the simulation options into local variables, casting to the appropriate data type
   #where necessary
   #stockname - a general comment describing the simulation
   stockname<-nodeval(root,"stockname","")
   if (debug==1) {cat("stockname=",stockname,"\n")}

   #indata.path - the folder where the input data files are stored
   #indata.path<-nodeval(root,"indata_path","")
   indata.path<-paste(FPRESS.Home(),"\\indata\\",sep="")
   if (debug==1) {cat("indata.path=",indata.path,"\n")}

   #progdata.path - the folder where the output data files are stored
   #progdata.path<-nodeval(root,"progdata_path","")
   outdata.path<-paste(FPRESS.Home(),"\\outdata\\",sep="")
   #if (debug==1) {cat("progdata.path=",progdata.path,"\n")}
   if (debug==1) {cat("outdata.path=",outdata.path,"\n")}

   #datfile - the name of the data file containing the initialisation data profiles
   datfile<-nodeval(root,"datfile","")
   if (debug==1) {cat("datfile=",datfile,"\n")}

   #recfile - the name of the file containing the historical recruitment data   
   recfile<-nodeval(root,"recfile","")
   if (debug==1) {cat("recfile=",recfile,"\n")}

   #catchfile - the name of the file containing the historical catch data
   catchfile<-nodeval(root,"catchfile","")
   if (debug==1) {cat("catchfile=",catchfile,"\n")}
   
   #minage - the minimum age group for the stock in the simulation
   minage<-as.integer(nodeval(root,"minage",1))
   if (debug==1) {cat("minage=",minage,"\n")}

   #maxage - the maximum age group for the stock in the simulation (treated as a plus group)
   maxage<-as.integer(nodeval(root,"maxage",1))
   if (debug==1) {cat("maxage=",maxage,"\n")}

   #fbarmin - the youngest age class to be used in the calculation of fbar
   fbarmin<-as.integer(nodeval(root,"fbarmin",0))
   if (debug==1) {cat("fbarmin=",fbarmin,"\n")}

   #fbarmax - the oldest age class to be used in the calculation of fbar
   fbarmax<-as.integer(nodeval(root,"fbarmax",1))
   if (debug==1) {cat("fbarmax=",fbarmax,"\n")}

   #ssbtest1 - virtual ssb test level 1 (often corresponding to Blim)
   ssbtest1<-as.integer(nodeval(root,"ssbtest1",0))
   if (debug==1) {cat("ssbtest1=",ssbtest1,"\n")}

   #ssbtest2 - virtual ssb test level 2 (often corresponding to Bpa)
   ssbtest2<-as.integer(nodeval(root,"ssbtest2",0))
   if (debug==1) {cat("ssbtest2=",ssbtest2,"\n")}

   #resolution
   resolution<-as.integer(nodeval(root,"resolution",0))
   if (debug==1) {cat("resolution=",resolution,"\n")}

   #startyear - the start year for the simulation
   startyear<-as.integer(nodeval(root,"startyear",1))
   if (debug==1) {cat("startyear=",startyear,"\n")}

   #years - the number of years the simulation is to run for
   years<-as.integer(nodeval(root,"years",1))
   if (debug==1) {cat("years=",years,"\n")}

   #iterations - the number of iterations to be performed for each TAC/F value
   iterations<-as.integer(nodeval(root,"iterations",1))
   if (debug==1) {cat("iterations=",iterations,"\n")}

   #histrec - the historical recruitment flag. Can only be 0 or 1. If 1 then historical recruitment
   #values as given in the recfile are used, otherwise not.
   histrec<-as.integer(nodeval(root,"histrec",0))
   if (debug==1) {cat("histrec=",histrec,"\n")}

   #histreccv - the coefficient of variation to be applied when adding in noise to the 
   #historical recruitment values. Only used when histrec=1
   histreccv<-as.numeric(nodeval(root,"histreccv",0))
   if (debug==1) {cat("histreccv=",histreccv,"\n")}

   #histcatch - the historical catch flag. Can only be 0 or 1. If 1 then the historical catch
   #values as given in the catchfile are used, otherwise not.
   histcatch<-as.integer(nodeval(root,"histcatch",0))
   if (debug==1) {cat("histcatch=",histcatch,"\n")}

   #histcatchcv - the coefficient of variation to be applied when adding noise to the
   #historical catch values. Only used when histcatch=1
   histcatchcv<-as.numeric(nodeval(root,"histcatchcv",0))
   if (debug==1) {cat("histcatchcv=",histcatchcv,"\n")}

   #rec - the recruitment model to be used
   rec<-as.integer(nodeval(root,"rec",0))
   if (debug==1) {cat("rec=",rec,"\n")}

   #recmean - the mean recruitment values (in thousands). Only used for some of the recruitment 
   #models (see recruit.r).
   recmean<-as.numeric(nodeval(root,"recmean",0))
   if (debug==1) {cat("recmean=",recmean,"\n")}

   #reccv - the coefficient of variation to be used when adding in noise to the caluclated
   #recruitment value. Only used for some of the recruitment models (see recruit.r).
   reccv<-as.numeric(nodeval(root,"reccv",0))
   if (debug==1) {cat("reccv=",reccv,"\n")}

   #seggrad - the gradient of the recruitment vs SSB line on a stock-recruitment diagram. Only
   #used for some of the recruitment models (see recruit.r).
   seggrad<-as.numeric(nodeval(root,"seggrad",1))
   if (debug==1) {cat("seggrad=",seggrad,"\n")}

   #ssbcut - the SSB cutoff point to use when calculating the number of recruits. Only used for some
   #of the recruitment models (see recruit.r).
   ssbcut<-as.numeric(nodeval(root,"ssbcut",0))
   if (debug==1) {cat("ssbcut=",ssbcut,"\n")}

   #datarecyear - the final year of historical recruitment data contained in recfile that is to be 
   #included in the simulation (including the bootstrapping options). This parameter permits the use of 
   #only a portion of the historical recruitment data. If greater than the final year's data then all 
   #data in the file will be used
   datarecyear<-as.integer(nodeval(root,"datarecyear",0))
   if (debug==1) {cat("datarecyear=",datarecyear,"\n")}
   
   #variable - indicates whether the fishery should be TAC controlled (0) or F controlled (1)
   variable<-as.numeric(nodeval(root,"variable",0))
   if (debug==1) {cat("variable=",variable,"\n")}

   #varcv - the coefficient of variation to used when adding in noise to the controlling variable
   varcv<-as.numeric(nodeval(root,"varcv",0))
   if (debug==1) {cat("varcv=",varcv,"\n")}
   
   #varbias - the amount of bias to apply when randomising the controlling variable
   varbias<-as.numeric(nodeval(root,"varbias",1))
   if (debug==1) {cat("varbias=",varbias,"\n")}

   #varmin - the minimum (first) or only (for resolution=0) value to use for the controlling variable
   varmin<-as.numeric(nodeval(root,"varmin",1))
   if (debug==1) {cat("varmin=",varmin,"\n")}

   #varmax - for simulations with resolution>0, he maximum (final) value to use for the controlling variable
   varmax<-as.numeric(nodeval(root,"varmax",1))
   if (debug==1) {cat("varmax=",varmax,"\n")}

   #assessdelay - this parameter controls the timing of assessment events. The assessment will be based
   #on simulated data corresponding to the current year less the number of years given in assessdelay.
   assessdelay<-as.integer(nodeval(root,"assessdelay",1))
   if (debug==1) {cat("assessdelay=",assessdelay,"\n")}
   
   #ssbassessbias - the amount of bias to apply to the ssb value during the assessment
   ssbassessbias<-as.numeric(nodeval(root,"ssbassessbias",1))
   if (debug==1) {cat("ssbassessbias=",ssbassessbias,"\n")}
   
   #ssbassesscv - the coefficient of variation to use with the ssb during the assessment
   ssbassesscv<-as.numeric(nodeval(root,"ssbassesscv",0))
   if (debug==1) {cat("ssbassesscv=",ssbassesscv,"\n")}
   
   #fbarassessbias - the amount of bias to apply to F during the assessment
   fbarassessbias<-as.numeric(nodeval(root,"fbarassessbias",1))
   if (debug==1) {cat("fbarassessbias=",fbarassessbias,"\n")}
   
   #fbarassesscv - the coefficient of variation to use with F during the assessment
   fbarassesscv<-as.numeric(nodeval(root,"fbarassesscv",0))
   if (debug==1) {cat("fbarassesscv=",fbarassesscv,"\n")}
   
   #hcrrule - an integer value corresponding to the appropriate harvest control rule function (see hcr.r)
   hcrrule<-as.integer(nodeval(root,"hcrrule",0))
   if (debug==1) {cat("hcrrule=",hcrrule,"\n")}
   
   #hcrperiod - the period (in years) between harvest control rule actions being applied to the assessment
   hcrperiod<-as.integer(nodeval(root,"hcrperiod",1))
   if (debug==1) {cat("hcrperiod=",hcrperiod,"\n")}
   
   #hcrchange - the proportional change to TAC of F (as a result of an HCR) permitted in a single year
   hcrchange<-as.numeric(nodeval(root,"hcrchange",1))
   if (debug==1) {cat("hcrchange=",hcrchange,"\n")}
   
   #ssbhcr - reference SSB level used by some HCRs for comparison with simulated SSB levels
   ssbhcr<-as.numeric(nodeval(root,"ssbhcr",0))
   if (debug==1) {cat("ssbhcr=",ssbhcr,"\n")}

   #fbarhcr - reference fbar value used by some HCRs for comparison with simulated fbar values
   fbarhcr<-as.numeric(nodeval(root,"fbarhcr",0))
   if (debug==1) {cat("fbarhcr=",fbarhcr,"\n")}  # EINAR - replaced ssbhcr with fbarhcr

   #minTAC - if a minimum TAC must be taken then this value will be greater than zero
   #applies only with variable=0 
   mintac<-as.numeric(nodeval(root,"minimumtac",0))
   if (debug==1) {cat("minimumtac=",minimumtac,"\n")}   # EINAR - replaced ssbhcr with minimumtac

   #output objects
   #ftac multiplier
   opftacmult<-as.numeric(nodeval(root,"opftacmult",0))
   #if (opftacmult==1) {savedobjs<-c(savedobjs,"FTacMult")}  EINAR - redundnant, see line 46
   
   #recruits
   oprecruits<-as.numeric(nodeval(root,"oprecruits",1))
   #if (oprecruits==1) {savedobjs<-c(savedobjs,"Recruits")}  EINAR - redundnant, see line 46

   #pop
   oppop<-as.numeric(nodeval(root,"oppop",0))
   #if (oppop==1) {savedobjs<-c(savedobjs,"Pop")}  EINAR - redundnant, see line 46

   #opftac
   opftac<-as.numeric(nodeval(root,"opftac",0))
   #if (opftac==1) {savedobjs<-c(savedobjs,"FTac")}    EINAR - redundnant, see line 46

   #opf
   opf<-as.numeric(nodeval(root,"opf",0))
   #if (opf==1) {savedobjs<-c(savedobjs,"FishingMortality")}  EINAR - redundnant, see line 46
   
   #opfmult
   opfmult<-as.numeric(nodeval(root,"opfmult",0))
   #if (opfmult==1) {savedobjs<-c(savedobjs,"FishingMortalityMult")}   EINAR - redundnant, see line 46

   #opspawningmass
   opspawningmass<-as.numeric(nodeval(root,"opspawningmass",1))
   #if (opspawningmass==1) {savedobjs<-c(savedobjs,"NewSpawnMass")}  EINAR - redundnant, see line 46

   #opfbar
   opfbar<-as.numeric(nodeval(root,"opfbar",1))
   #if (opfbar==1) {savedobjs<-c(savedobjs,"FBar")}  EINAR - redundnant, see line 46

   #opcatch
   opcatch<-as.numeric(nodeval(root,"opcatch",0))
   #if (opcatch==1) {savedobjs<-c(savedobjs,"Catch")}  EINAR - redundnant, see line 46

   #opcatchweight
   opcatchweight<-as.numeric(nodeval(root,"opcatchweight",1))
   #if (opcatchweight==1) {savedobjs<-c(savedobjs,"CatchWeight")}  EINAR - redundnant, see line 46

   #opyestock
   opyestock<-as.numeric(nodeval(root,"opyestock",0))
   #if (opyestock==1) {savedobjs<-c(savedobjs,"YearEndStock")}  EINAR - redundnant, see line 46

   #opindex
   opindex<-as.numeric(nodeval(root,"opindex",0))

   savedobjs<-c()

   if (opindex>=1024) {
      savedobjs<-c(savedobjs,"YearEndStock")
      opindex<-opindex-1024
   }

   if (opindex>=512) {
      savedobjs<-c(savedobjs,"CatchWeight")
      opindex<-opindex-512
   }

   if (opindex>=256) {
      savedobjs<-c(savedobjs,"Catch")
      opindex<-opindex-256
   }

   if (opindex>=128) {
      savedobjs<-c(savedobjs,"FBar")
      opindex<-opindex-128
   }

   if (opindex>=64) {
      savedobjs<-c(savedobjs,"NewSpawnMass")
      opindex<-opindex-64
   }
   
   if (opindex>=32) {
      savedobjs<-c(savedobjs,"FishingMortalityMult")
      opindex<-opindex-32
   }

   if (opindex>=16) {
      savedobjs<-c(savedobjs,"FishingMortality")
      opindex<-opindex-16
   }

   if (opindex>=8) {
      savedobjs<-c(savedobjs,"FTac")
      opindex<-opindex-8
   }

   if (opindex>=4) {
      savedobjs<-c(savedobjs,"Pop")
      opindex<-opindex-4
   }

   if (opindex>=2) {
      savedobjs<-c(savedobjs,"Recruits")
      opindex<-opindex-2
   }

   if (opindex>=1) {
      savedobjs<-c(savedobjs,"FTacMult")
      opindex<-opindex-1
   }

   #if (bitAnd(opindex,1)==1) {savedobjs<-c(savedobjs,"FTacMult")}
   #if (bitAnd(opindex,2)==2) {savedobjs<-c(savedobjs,"Recruits")}
   #if (bitAnd(opindex,4)==4) {savedobjs<-c(savedobjs,"Pop")}
   #if (bitAnd(opindex,8)==8) {savedobjs<-c(savedobjs,"FTac")}
   #if (bitAnd(opindex,16)==16) {savedobjs<-c(savedobjs,"FishingMortality")}
   #if (bitAnd(opindex,32)==32) {savedobjs<-c(savedobjs,"FishingMortalityMult")}
   #if (bitAnd(opindex,64)==64) {savedobjs<-c(savedobjs,"NewSpawnMass")}
   #if (bitAnd(opindex,128)==128) {savedobjs<-c(savedobjs,"FBar")}
   #if (bitAnd(opindex,256)==256) {savedobjs<-c(savedobjs,"Catch")}
   #if (bitAnd(opindex,512)==512) {savedobjs<-c(savedobjs,"CatchWeight")}
   #if (bitAnd(opindex,1024)==1024) {savedobjs<-c(savedobjs,"YearEndStock")}

   if (debug==1) {cat("savedobjs=",savedobjs,"\n")}

   #construct the fully qualified filenames for the input data file, the historical recruitment data file
   #and the historical catches file
   #if (substr(indata.path,nchar(indata.path),nchar(indata.path))=="\\") {
   #   datainfile<-paste(indata.path,datfile,sep="") } else 
   #   recruitinfile<-paste(indata.path,recfile,sep="")
   #   catchinfile<-paste(indata.path,catchfile,sep="")
   #{
   #   datainfile<-paste(indata.path,"\\",datfile,sep="")
   #   recruitinfile<-paste(indata.path,"\\",recfile,sep="")
   #   catchinfile<-paste(indata.path,"\\",catchfile,sep="")
   #}

   datainfile<-datfile
   recruitinfile<-recfile
   catchinfile<-catchfile

   if (debug==1) {cat("datainfile=",datainfile,"\n")}
   if (debug==1) {cat("recruitinfile=",recruitinfile,"\n")}
   if (debug==1) {cat("catchinfile=",catchinfile,"\n")}

   ############################################################################################################
   #####################----------SIMULATION EXECUTION-----------------------##################################
   ############################################################################################################

   #set the current path for R to the progdata.path (i.e. the location of the F-PRESS .r files)
   #setwd(progdata.path)

   #print some information to the screen & store the start time in a local variable.
   #on completion of the simulation this can be compared with the system time
   #to calculate the elapsed time for the run.
   print(stockname)
   print(Sys.time())
   startdate<-as.POSIXlt(Sys.time(),Sys.timezone())

   #create a vector of output filenames. The number of output files will be resolution+1
   FLRfiles<-paste("FLRObjects_",runref,"_",1:(resolution+1),".dat",sep="")

   ##initialise the local variable ftac to the minimum value of the controlling variable (varmin)
   ftac<-varmin

   #the ftac value used to control the fishery is modified in various ways e.g. by the assessment and management
   #module. This is usually done by calculating a multiplier. Here, a couple of local variables used to store
   #the multiplier are declared and initialised to unity.
   #the multiplier
   ftacmult<-1
   #temporal storage
   oldftacmult<-1
 
   #initialise a variable to hold the final year for the projection
   endyear<-startyear+years-1

   #check initialisation data file exists
   if (!(file.exists(datainfile))) {
      stop(paste("\nInitialisation file",datainfile,"doesn't exist\nUnable to continue\n",sep=" "))
   }

   #read in the initialisation data from file datainfile
   indata<-matrix(scan(datainfile,0,quiet=TRUE),ncol=17,byrow=TRUE)   # EINAR - replaced ncol=18 with ncol=17
   #the number of distinct age classes.....
   numage<-length(indata[,1])

   #check that the number of ages in the initialistion data file matches the range given
   #by minage - maxage (the initialisation data file age limits)
   if ((maxage-minage+1)!=numage) {
      stop(paste("\nThe age class data supplied in the initialisation data file (",datainfile,")\ndoes not match the age class range (",minage,"-",maxage,") in the options file (",optionsfile,")\n",sep=""))
   }

   #dimension a number of arrays to store the various parameters of interest. Each array will hold all values i.e. over all age groups, 
   #simulations years and iterations for each resolution. Initial values are all zero
   #the arrays are designed to look like FLQuant objects, hence the age, year, unit, season and area dimension names and some redundancy
   #(with unit and season). The area dimension is being used in this context as the iteration dimension

   #temporary arr_dim vector used to initialise arrays
   arr_dim<-c(numage,years,1,1,iterations)
   #temporary arr_dimnames list
   arr_dimnames=list(age=minage:maxage,year=startyear:endyear,unit='unique',season='all',area=1:iterations)

   catchweightarray<-array(0,dim=arr_dim,dimnames=arr_dimnames)
   catcharray<-array(0,dim=arr_dim,dimnames=arr_dimnames)
   newspawnmassarray<-array(0,dim=arr_dim,dimnames=arr_dimnames)
   yearendstockarray<-array(0,dim=arr_dim,dimnames=arr_dimnames)
   poparray<-array(0,dim=arr_dim,dimnames=arr_dimnames)
   fishingmortalityarray<-array(0,dim=arr_dim,dimnames=arr_dimnames)
   fishingmortalitymultarray<-array(0,dim=arr_dim,dimnames=arr_dimnames)
   farray<-array(0,dim=arr_dim,dimnames=arr_dimnames)

   #the following arrays do not include age-disaggregated data and so do not expand this dimension
   arr_dim_noage<-c(1,years,1,1,iterations)
   arr_dimnames_noage<-list(age=0,year=startyear:endyear,unit='unique',season='all',area=1:iterations)

   recruitsarray<-array(0,dim=arr_dim_noage,dimnames=arr_dimnames_noage)
   fbararray<-array(0,dim=arr_dim_noage,dimnames=arr_dimnames_noage)
   ftacarray<-array(0,dim=arr_dim_noage,dimnames=arr_dimnames_noage)
   ftacmultarray<-array(0,dim=arr_dim_noage,dimnames=arr_dimnames_noage)
   hcrsubarray<-array(0,dim=arr_dim_noage,dimnames=arr_dimnames_noage)

   #read in historical recruitment data ...
   input<-scan(recruitinfile,list(0,0,0),quiet=TRUE)

   #...and assign to local vectors
   recyear<-input[[1]]   #recruitment data year vector
   recr<-input[[2]]      #recruitment data recruitment numbers vector
   rssbin<-input[[3]]    #recruitment data ssb vector

   #initialise some variables to hold the length of the historical recruitment dataset
   #(lenrecyr) and the final year for which data is available (maxrecyr). In the event
   #that this data is not to be used set the maximum year to 0

   lenrecyr<-length(recyear)
   maxrecyr<-0

   if (histrec==1) {maxrecyr<-max(recyear)}

   #calculate the number of year between the start of the simulation and the final year of recruitment
   #data and store it in local variable diffxs
   
   diffxs<-max(recyear)-startyear+1
  
   #create three new vectors to hold the recruitment data offset so that the recruitment value corresponds to the year that produced 
   #this age-class (the spawning year) rather than the year it entered the fishery. Depending on the value of minage, this is likely to 
   #require recruitment data for years that are after the start date of the projection (e.g. if the simulation starts in 2000 and minage = 2, 
   #then recruitment data will be required for at least 1998, 1999, 2000 and 2001). 
  
   spwnyr<-seq(1,1,length=lenrecyr-max(minage,diffxs))
   spwnssb<-spwnyr
   spwnrec<-spwnyr

   for(i in 1:(lenrecyr-max(minage,diffxs))){
      spwnyr[i]<-recyear[i]
      spwnssb[i]<-rssbin[i]
      spwnrec[i]<-recr[i+minage]
   }

   #These lines are similar to the above - vectors are set up with the historical recruitment data and the vector spwnfrec is offset so 
   #that it corresponds to the spawning year rather than the year of entry to the fishery. However, the vectors may be of a different 
   #length than those above depending on the value of datarecyear. This parameter gives the maximum year in the recfile that 
   #data is to be used from in the recruitment functions. This allows fictional future recruitment to be entered into the recruits data file, 
   #but only the real historical data to be used in fitting parametric functions or bootstrapping. If datarecyear > max(recyear) 
   #(the oldest year in the recruits data file), then all data in the file is used in the recruitment functions. The vectors spwnfnyr,spwnfnssb,
   #spwnfnrec are used in the recruitment functions

   #calculate the number of years between the last year of recruitment data and the final year that is to be used in the simulations
   diffxt<-max(recyear)-datarecyear+1

   spwnfnyr<-seq(1,1,length=lenrecyr-max(minage,diffxt))
   spwnfnssb<-spwnfnyr
   spwnfnrec<-spwnfnyr

   for(i in 1:(lenrecyr-max(minage,diffxt))){
     spwnfnyr[i]<-recyear[i]
     spwnfnssb[i]<-rssbin[i]
     spwnfnrec[i]<-recr[i+minage]
   }

   #read in historical catch data
   input<-scan(catchinfile,list(0,0,0),quiet=TRUE)
   
   #and assign to local vectors for
   #year,
   catyear<-input[[1]]
   #catch &
   ccatch<-input[[2]]
   #fbar
   cfbar<-input[[3]]

   if(histcatch==1){
      #last year of catch data
      maxcatyr<-max(catyear)
      #number of years of catch data
      lencatyr<-length(catyear)
   } else  {
      #set maximum year to zero
      maxcatyr<-0
   }

   #create initialisation vectors using data from datafile

   #initialise vars with data from datinfile

   #age class
   initage<-indata[,1]
   #population (000s)
   initpopint<-indata[,2]
   #population cv
   initpopcv<-indata[,3]
   #spawning weight (kg)
   initspwnweightint<-indata[,4]
   #spawning weight cv
   initspwnweightcv <-indata[,5]
   #catch weight (kg)
   initcatweightint<-indata[,6]
   #catch weight cv
   initcatweightcv<-indata[,7]
   #maturity
   initmatureint<-indata[,8]
   #maturity cv
   initmaturecv<-indata[,9]
   #fishing mortality
   initfint<-indata[,10]
   #fishing mortality cv
   initfcv<-indata[,11]
   #proportion of fishing mortality before spawning
   initfprop<-indata[,12]
   #discard mortality
   initdisfint<-indata[,13]
   #discard mortality cv
   initdisfcv<-indata[,14]
   #natural mortality
   initmint<-indata[,15]
   #natural mortality cv
   initmcv<-indata[,16]
   #proportio of natural mortality before spawning
   initmprop<-indata[,17]

   #################################################
   #START OF MAIN BODY##############################
   #################################################

   #################################################
   #loop over values of F,TAC as appropriate########
   #res is the counter variable#####################
   #################################################
   for(res in 0:resolution) {

      if (debug==1) {cat("res=",res,"\n")}

      #calculate the appropriate FTAC value for this resolution
      if (resolution>0) {

	 ftac<-((varmax-varmin)*res/resolution)+varmin

	 #ensure we don't proceed with a 0 value
	 if(ftac==0){ftac<-0.0001}

	 #calculate the percentage of the simulation that is complete
	 #for reporting purposes
	 percent<-round((res/(resolution+1))*100,digits=2)

	 if (variable==1) {
	    cat("fval = ",ftac," ",percent,"% complete","\n",sep="")
	 } else {
	    cat("tacval = ",ftac," ",percent,"% complete","\n",sep="")
	 }
      }

      #update the console to ouput above messages
      flush.console()

      ########################################################  
      #main iterative loop####################################
      #iter is the counter variable###########################
      ########################################################
      for(iter in 1:iterations){

	 #if the resolution is zero then the simulation is running for a single 
	 #TAC or F value. In this case report progress on basis of the number of iterations
	 #completed. Issue report every 10%
	 if (resolution == 0) {
	    percent <- 100*iter/iterations
	    if ((percent%%10) == 0) {
	       cat(percent,"% complete","\n",sep="")
	    }
	 }

	 flush.console()

	 if (debug==1) {cat("iter=",iter,"\n")}

	 ############################################################
	 #year loop##################################################
	 #year is the counter variable###############################
	 ############################################################
	 for(year in 1:years){
    
	    if (debug==1) {cat("year=",year,"\n")}

	    #calculate the current simulation year
	    simyear<-year+startyear-1

	    if (debug==1) {cat("A&M..")}

	    ########assessment & management#############################
	    if((year-assessdelay)>0){

	       #apply the assessment model to calculate new vales for ssb and fbar (see assessment.r for details)
	       assres<-assessment(ssb,
	                          fbar,
				  ssbassessbias,
				  ssbassesscv,
				  fbarassessbias,
				  fbarassesscv)

	       #ssb is the first element...
	       ssb<-assres[1]
	       #...and fbar the second
	       fbar<-assres[2]
	       
	       #pass the modified ssb and fbar values to the management routine which will calculate a ftac multiplier via the HCR
	       man_ret<-management(ssb,
	                           fbar,
				   ftac,
				   year,
				   years,
				   assessdelay,
				   hcrperiod,
				   hcrrule,
				   hcrchange,
				   ssbhcr,
				   oldftacmult,
				   fbarhcr,
				   ssbtest1,
				   ssbtest2,
				   mintac)

	       #the multiplier is returned as the first element...
	       ftacmult<-man_ret[1]
	       #...and a second value of interest can be returned as the second (not currently used)
	       hcrsub<-man_ret[2]

	    } else {

	       #the multiplier is simply set to unity so no change is effected
	       ftacmult<-1
	       hcrsub<-0
	    
	    }

	    #update the local ftacmult and hcrsub arrays using the year and iter counters to assign the values
	    #to the correct slots
	    ftacmultarray[1,year,1,1,iter]<-ftacmult
	    hcrsubarray[1,year,1,1,iter]<-hcrsub

	    #apply the multiplier to calculate the new ftac, 
	    #first saving the original value in temporary variable origftac
	    origftac<-ftac

	    #check here to see if a minimum TAC must be taken
	    #if so, ignore the multiplier and impose the minimum TAC condition
	    #should move this functionality into the management module?
	    if ((variable==0) & (ftacmult*ftac < mintac)) {
	       ftac=mintac
	    } else {
	       ftac<-ftacmult*ftac
	    }
    
	    if (debug==1) {cat("Init..")}

	    #initialise vectors for start of year
	    #age class
	    age<-initage
	    #population. If past the first year of the simulation, use the population as simulated 
	    #at the end of the previous year. If in the first year of the simulation, used the populations
	    #as specified in the datafile
	    if (year>1) {popint<-newyearplus1} else {popint<-initpopint}
	    #population cv
	    popcv<-initpopcv
	    #spawning weight
	    spwnweightint<-initspwnweightint
	    #spawning weight cv
	    spwnweightcv <-initspwnweightcv
	    #catch weight
	    catweightint<-initcatweightint
	    #catch weight cv
	    catweightcv<-initcatweightcv
	    #maturity
	    matureint<-initmatureint
	    #maturity cv
	    maturecv<-initmaturecv
	    #fishing mortality
	    fint<-initfint
	    #fishing mortality cv
	    fcv<-initfcv
	    #proportion of fishing mortality prior to spawning
	    fprop<-initfprop
	    #discard mortality
	    disfint<-initdisfint
	    #discard mortality cv
	    disfcv<-initdisfcv
	    #natural mortality
	    mint<-initmint
	    #natural mortality cv
	    mcv<-initmcv
	    #proportion of natural mortality before spawning
	    mprop<-initmprop

	    #cat("Starting SSB = ",sum(spwnweightint*initpopint*matureint),"\n")

	    #recruits entering fishery
	    #population will only be supplemented here if minage>0. When minage=0 the fish enter immediately
	    #after spawning (see later code)

	    if (debug==1) {cat("Rec after spawn..")}

	    if (minage>0) {

	       #we also must have some historical recruitment values here since entry to the fishery
	       #is after spawning and the model execution period may not cover the year for which the
	       #recruitment is originally calculated. For example, if minage=3 and the model starts
	       #execution in 2000, the first year that the model can calculate recruitment for is 2003.
	       #thus historical values are required for 2000, 2001 and 2002

	       #if (histrec==1) {

		  #if (simyear <= maxrecyr) {			  #BUG - should be simyear-minage to properly offset data in SSB/R file
		  if (simyear-minage <= maxrecyr) {

		     #loop through historical data to find the value we need
		     for (i in 1:lenrecyr) {
			#if (recyear[i]==simyear) {index<-i}		#BUG - should be simyear-minage to properly offset data in SSB/R file
			if (recyear[i]==simyear-minage) {index<-i}
		     }

		     #initialise value to -1
		     frec<--1

		     #apply historical recruitment cv value
		     while(frec<0){frec<-rnorm(1,recr[index],recr[index]*histreccv)}

		     recruitsarray[1,year,1,1,iter]<-frec
		     fishrecruited<-frec

	       
		  } else {

		     #simulation year has passed end of historical value dataset and
		     #recruitment value should already be in the recruitsarray
		     frec<-as.numeric(recruitsarray[1,(year-minage),1,1,iter])

		  }

		 fishrecruited<-frec 

	       #}


	    }


            #Randomising parameters - adding noise / stochasticity to account for environmental/operational variability

            #Initial population (randomised in first year of projection only)
	    pop<-popint 
            popintsd<-(popcv*popint)
   
	    if(year==1){ 
	       pop<--popint
	       #keep drawing until a positive result is obtained
	       while(min(pop)<0){
		  for(i in 1:numage){
		     pop[i]<-rnorm(1,popint[i],popintsd[i])
		  }
	       } 
	    }

	    #if minage>0 then take the fishrecruited value calculated above for the first element
	    #of the population vector
	    if(minage>0){pop[1]<-fishrecruited}

	    #Spawning weight
	    spwnweight<--spwnweightint
	    spwnweightsd<-spwnweightcv*spwnweightint 

	    while(min(spwnweight)<0){
	       for(i in 1:numage){
		  spwnweight[i]<-rnorm(1,spwnweightint[i],spwnweightsd[i])
	       }
	    } 

	    #Catch weight
	    catweight<--catweightint
	    catweightsd<-catweightcv*catweightint 

	    while(min(catweight)<0){
	       for(i in 1:numage){
		  catweight[i]<-rnorm(1,catweightint[i],catweightsd[i])
	       } 
	    } 
	    
	    #Maturity
	    mature<--matureint
	    maturesd<-maturecv*matureint

	    while(min(mature)<0){
	       for(i in 1:numage){
		  mature[i]<-min(rnorm(1,matureint[i],maturesd[i]),1)
	       }
	    } 

	    #Fishing mortality
	    f<--fint
	    fsd<-fcv*fint
      
	    while(min(f)<0){ 
	       for(i in 1:numage){
		  f[i]<-rnorm(1,fint[i],fsd[i])
	       }
	    } 

	    #Discard mortality
	    disf<--disfint
	    disfsd<-disfcv*disfint
       
	    while(min(disf)<0){ 
	       for(i in 1:numage){
		  disf[i]<-fmult*rnorm(1,disfint[i],disfsd[i])
	       }
	    } 
                  
	    #Natural mortality
	    m<--mint
	    msd<-mcv*mint

	    while(min(m)<0){
	       for(i in 1:numage){
		  m[i]<-rnorm(1,mint[i],msd[i])
	       }
	    } 

	    #newfbar - calculated by taking the mean of f over the age groups defined by the parameters fbarmin and fbarmax
	    newfbar<-mean(f[(fbarmin-minage+1):(fbarmax-minage+1)])

	    if (debug==1) {cat("ftac..")}

	    #Calculate FTAC
	    #If historical catches are to be used and the current simulation year is within the available
	    #data period then find the appropriate catch or fbar from the historical dataset. This value
	    #will then be used as the mean value for the normal distribution from which a randomised value
	    #for ftac will be drawn with the cv value given by histcatchcv

	    #if historical values are used for catch then the bias and cv associated with 
	    #F/TAC will be set to 1 and 0 respectively
	    #and 0 respectively

	    newvarcv<-varcv
	    newvarbias<-varbias

	    if ((histcatch==1) & (simyear<=maxcatyr)) {

	       #identify the relevant year
	       index<-0
	       for (i in 1:lencatyr) {
		  if(catyear[i]==simyear){index<-i}
	       }

	       #now perform the random draw
          
	       #F controlled fishery 
	       if (variable==1) {
		  #initialise
		  ftac<--1
		  #randomise
		  while(ftac<0){ftac<-rnorm(1,cfbar[index],cfbar[index]*histcatchcv)} 
		  #reset cv, bias parameters
		  newvarcv<-0
		  newvarbias<-1
	       }
            
	       #TAC controlled fishery
	       if (variable==0) { 
		  #initialise
		  ftac<--1
		  #randomise
		  while(ftac<0){ftac<-rnorm(1,ccatch[index],ccatch[index]*histcatchcv)} 
		  #reset cv, bias parameters
		  newvarcv<-0
		  newvarbias<-1
	       } 
	    } 

	    #update the ftac array
	    ftacarray[1,year,1,1,iter]<-ftac

	    if (debug==1) {cat("Fmult..")}

	    #Apply F multiplier to F vectors (only required if simulation is based on f)
	    #The specified level of F given by ftac is altered by possible bias (overfishing) given by varbias 
	    #and then randomised by drawing from a Normal distribution. The randomised value of F (freq) is the 
	    #required fishing mortality for this year. A multiplier, fmult, is calculated from newfbar so that 
	    #when multiplied by the current F vector f it will give the required value of freq. Finally, the F 
	    #vector f is multiplied by this multiplier along with the discard F vector disf.
	    #In reality, discard F may not change in the same way as F when catches are reduced.
      
	    if(variable==1){
	       #apply bias
	       fnewval <- ftac * newvarbias   # EINAR - replaced newfbias with newvarbias

	       #randomise fnewval
	       nf<--1        
	       while(nf<0){nf<-rnorm(1,fnewval,fnewval*newvarcv)}
    
	       #this is the required f vector
	       freq<-nf

	       #calculate f multiplier
	       fmult<-freq/newfbar
        
	       #apply multiplier to f
	       f<-fmult*f
	       #apply multiplier to discard f
	       disf<-disf*fmult
	    }


	    #If using a TAC, calculate the appropriate F multiplier.
	    #The specified level of TAC given by FTAC is altered by possible bias (overfishing) given by varbias and then 
	    #randomised (this occurs each year) by drawing from a Normal distribution. The randomised value of TAC given by 
	    #tac is the required catch for this year. Function ffactor is then used to calculate an F multiplier, ffac, 
	    #that when applied to the current F vector f, will produce the required catch equal to the TAC (so that fishing 
	    #mortality in the simulation is always actually calculated using F even when the virtual fishery uses TAC). Finally, 
	    #the F vector f is multiplied by this multiplier along with the discard F vector disf.

	    if(variable==0){
	       #apply bias
	       tacval<-ftac*newvarbias  #=1 no bias, <1 under, >1 over
      
	       #randomise
	       ntac<--1
	       while(ntac<0){ntac<-rnorm(1,tacval,tacval*newvarcv)}

	       #this is the requied tac vector
	       tac<-ntac

	       #calculate f vector
	       ffac<-ffactor(tac,pop,f,disf,catweight,m,numage)

	       #apply multipler to f
	       f<-f*ffac
	       #apply multiplier to discard f
	       disf<-disf*ffac
	       fmult<-ffac
	    }

	    #Update fishing mortality and fishing mortality multiplier arrays
	    fishingmortalityarray[,year,1,1,iter]<-f
	    fishingmortalitymultarray[,year,1,1,iter]<-fmult

	    ##################################################################
	    #POPULATION DYNAMICS
	    ##################################################################

	    #PRIOR TO SPAWNING

	    if (debug==1) {cat("Dyn Before Spawn..")} 

	    #proportion of fishing mortality that takes place before spawning
	    fbef<-(f+disf)*fprop
      
	    #proportion of fishing mortality that takes place after spawning
	    faft<-(f+disf)*(1-fprop)
      
	    #proportion of natural mortality that takes place before spawning
	    mbef<-m*mprop
      
	    #proportion of natural mortality that takes place after spawning
	    maft<-m-mbef

	    if (debug==1) {cat("fbef,mbef =",fbef,mbef,"..")}

	    #calculate the new population due to the fishing and natural mortality that occurs before spawning
	    newstock<-(pop*exp(-mbef-fbef))
      
	    #new spawning population is the total population multiplied by the maturity vector
	    newspawnpop<-(newstock*mature)
      
	    #new spawning biomass is the new spawning population multiplied by the spawning weight vector
	    newspawnmass<-newspawnpop*spwnweight 
      
	    #update the local array with the new spawning biomass for this iteration/year
	    newspawnmassarray[,year,1,1,iter]<-newspawnmass
      
	    #calculate the total spawning stock biomass by summing over all age classes
	    ssb<-sum(newspawnmass)
 
	    if (debug==1) {cat("Recruit..")} 

	    #RECRUITMENT
      
	    #calculate a value for the number of new recruits using the specific recruitment model. 
	    
	    #USER DEFINED RECRUITMENT MODELS - ass new recruitment model to switch construct.
	    recruitsnew <- switch(rec+1,
				  recruit0(recmean),
		                  recruit1(recmean,reccv),
			          recruit2(spwnfnyr,spwnfnrec),
			          recruit3(ssb,spwnfnyr,spwnfnrec,spwnfnssb,ssbcut),
				  recruit4(ssb,spwnfnrec,spwnfnssb),
				  recruit5(ssb,spwnfnrec,spwnfnssb),
                                  recruit6(ssb,ssbcut,recmean,seggrad),
                                  recruit7(ssb,ssbcut,recmean,seggrad,reccv))

	    #if the minimum age is zero then the fish enter the fishery from age 0 
	    #i.e. new recruits enter the fishery immediately after spawning so we add them now
	    if(minage==0){                      
      
	       if(simyear > maxrecyr){         
        
		  #If the current simulation year is greater than the oldest year in the recruitment data file, 
		  #then set the recruitment (frec) = recruitsnew
		  frec<-recruitsnew
        
	       } else {
        
		  #if the current simulation year is contained in the recruitment data file
		  #then the expected recruitment is set to the corresponding value given in the file
        
		  for(i in 1:lenrecyr){
		     if(recyear[i]==simyear){num<-i}
		  }
        
		  #randomise the number of recruits
		  frec<--1
		  while(frec<0){frec<-rnorm(1,recr[num],recr[num]*histreccv)}
        
	       }
        
	       #assign the new recruits to the first population vector element
	       newstock[1]<-frec 
	       yearrecruits<-frec

	       #write data to local recruitsarray
	       recruitsarray[1,year,1,1,iter]<-yearrecruits

	    } else {
	 
	       #the value recruitsnew applies to a future year
	       #write this into a future slot in the recruitsarray (provided there is one - no point in writing beyond end of simulation period)
	       if ((year+minage) <= years) {
		  recruitsarray[1,year+minage,1,1,iter]<-recruitsnew
	       }
	    }

	    if (debug==1) {cat("Removing Catch..")}

	    #TAKE THE CATCH
	    fbarz<-c()
	    for(i in (fbarmin-minage+1):(fbarmax-minage+1)){
	       fbarz<-c(fbarz,f[i])
	    }

	    fbar<-mean(fbarz)

	    #update local array
	    fbararray[1,year,1,1,iter]<-fbar

	    #calculate the catch in number
	    catch<-(pop*f/(f+disf+m+0.00001)*(1-exp(-f-disf-m)))
      
	    #calculate the catch weight
	    catchwght<-catch*catweight
      
	    #update local array
	    catchweightarray[,year,1,1,iter]<-catchwght
      
	    if (debug==1) {cat("Dyn After Spawn\n")}

	    #AFTER SPAWNING
	    #calculate the year end population by applying the fishing and natural mortality that occurs after spawning
	    yearendstock<-newstock*exp(-maft-faft)
      
	    #update local arrays
	    yearendstockarray[,year,1,1,iter]<-yearendstock
	    poparray[,year,1,1,iter]<-pop
	    catcharray[,year,1,1,iter]<-catch
	    farray[,year,1,1,iter]<-f
      
	    #plus group
	    yearplus1<-0
	    for(i in 1:(numage-2)){
	       yearplus1<-c(yearplus1,yearendstock[i])
	    }
      
	    #add on the plus group
	    yearplus1<-c(yearplus1, yearendstock[numage-1]+yearendstock[numage])   # Plus group in last space

	    #save vals in temp vars for initialisation of population in next year
	    newyearplus1<-yearplus1

	    #Note that the output has original input variances NOT new variances calculated from the data
    
	    ftac<-origftac
     
	 }    
	 ###########################################################
	 #end of year loop##########################################
	 ###########################################################

      }   
      ##########################################################
      #end of main iterative loop###############################
      ##########################################################
 
      #Save data
      #Create FLQuant objects
      #FLQuant() is the creator function for FLQuant objects
      CatchWeight<-FLQuant(catchweightarray,units='Kgs')
      NewSpawnMass<-FLQuant(newspawnmassarray,units='Kgs')
      FBar<-FLQuant(fbararray,units='F')
      Recruits<-FLQuant(recruitsarray,units='000s')
      YearEndStock<-FLQuant(yearendstockarray,units='Kgs')
      Pop<-FLQuant(poparray,units='Kgs')
      Catch<-FLQuant(catcharray,units='Kgs')
      FishingMortality<-FLQuant(fishingmortalityarray,units='F')
      FishingMortalityMult<-FLQuant(fishingmortalitymultarray,units='F')
      F<-FLQuant(farray,units='F')
      FTac<-FLQuant(ftacarray,units='ftac')
      FTacMult<-FLQuant(ftacmultarray,units='mult')

      #save specified objects
      if (substr(outdata.path,nchar(outdata.path),nchar(outdata.path))=="\\") {
	 outfile<-paste(outdata.path,FLRfiles[res+1],sep="") } else
      {
	 outfile<-paste(outdata.path,"\\",FLRfiles[res+1],sep="")
      }

      save(list=savedobjs,file=outfile)

   }     
   ############################################################
   #end of resolution loop#####################################
   ############################################################

   #some end logging
   print(Sys.time())
   #calculate the end time and the difference with the starttime and
   #print duration of run to screen
   enddate<-as.POSIXlt(Sys.time(),Sys.timezone())
   diff<-difftime(enddate,startdate,tz="GMT",units="secs")
   cat("Execution Time:",diff[[1]]," secs", "\n")

   #restore working directory
   setwd(currwd)

# EOF
}
