#ssbriskplot.r
#Andrew Campbell
#Marine Institite, Galway
#andrew.campbell@marine.ie

ssbriskplot<-function(runref = "missing",
		      ssbtest1 = "missing",
		      ssbtest2 = "missing",
		      firstyear = "missing",
		      finalyear = "missing"){

   #runref - simulation reference
   #ssbtest1 - ssb test level 1
   #ssbtest2 - ssb test level2
   #firstyear - first year to include in risk calculation
   #finalyear - final year to include in risk calculation

   #save graphics settings
   def.par<-par(no.readonly=TRUE)

   #prompt for run reference
   if (missing(runref)) { 
      runref<-winDialogString("Run Reference?","")
   }

   if (is.null(runref)) {stop("Cancelled\n")}

   if (nchar(runref)==0) {stop("Exit - invalid run reference")}

   #options file
   optionsfile = paste(FPRESS.Home(),"\\options\\","options",runref,".xml",sep="")

   #check options file exists
   if (!(file.exists(optionsfile))) {
      stop(paste("\nOptions file",optionsfile,"doesn't exist\nUnable to continue\n",sep=" "))
   }

   #load xml tree
   doc<-xmlTreeParse(optionsfile)

   #root xmlNode object
   root<-xmlRoot(doc)

   #node values
   stockname <- nodeval(root,"stockname","")
   startyear <- as.integer(nodeval(root,"startyear","2000"))
   years <- as.integer(nodeval(root,"years",1))
   ssbtest1 <- as.integer(nodeval(root,"ssbtest1",0))
   ssbtest2 <- as.integer(nodeval(root,"ssbtest2",0))
   variable <- as.integer(nodeval(root,"variable",0))
   varmin <- as.numeric(nodeval(root,"varmin",0))
   varmax <- as.numeric(nodeval(root,"varmax",0))
   resolution <- as.integer(nodeval(root,"resolution",1))
   iterations <- as.integer(nodeval(root,"iterations",1))  

   if (resolution==0) {stop("invalid plot\n")}

   #read ssb test levels from options file if none supplied and prompt user
   if (missing(ssbtest1)) {
      ssbtest1 <- as.integer(nodeval(root,"ssbtest1",0))
   
      ssbtest1 <- winDialogString("SSBTest1?",as.character(ssbtest1))

      if (is.null(ssbtest1)) {stop("Cancelled\n")}
      if (nchar(ssbtest1)==0) {stop("Exit - invalid ssbtest1 parameter")}
   }

   if (missing(ssbtest2)) {
      ssbtest2 <- as.integer(nodeval(root,"ssbtest2",0))
   
      ssbtest2 <- winDialogString("SSBTest2?",as.character(ssbtest2))

      if (is.null(ssbtest2)) {stop("Cancelled\n")}
      if (nchar(ssbtest2)==0) {stop("Exit - invalid ssbtest2 parameter")}
   }

   if (missing(firstyear)) {
      firstyear <- winDialogString(paste("Enter first Year for risk calculation","(",
				         as.character(startyear),"-",as.character(startyear+years-1),")"),
					 as.character(startyear+years-1))

      if (is.null(firstyear)) {stop("Cancelled\n")}
      if (nchar(firstyear)==0) {stop("Exit - invalid firstyear parameter")}

   }

   if (missing(finalyear)) {
      finalyear <- winDialogString(paste("Enter final Year for risk calculation","(",
				         as.character(startyear),"-",as.character(startyear+years-1),")"),
					 as.character(startyear+years-1))

      if (is.null(finalyear)) {stop("Cancelled\n")}
      if (nchar(finalyear)==0) {stop("Exit - invalid finalyear parameter")}

   }

   #cast ssb/years to numerics
   ssbtest1 <- as.numeric(ssbtest1)
   ssbtest2 <- as.numeric(ssbtest2)
   firstyear <- as.numeric(firstyear)
   finalyear <- as.numeric(finalyear)
   years <- as.integer(finalyear - firstyear + 1)

   ssb1res<-c()
   ssb2res<-c()

   var <- seq(varmin,varmax,by=(varmax-varmin)/resolution)
   legx <- varmin

   xlabel <- "FBar"
   if (variable==0) {xlabel <- "TAC"}

   for (i in 1:(resolution+1)) {

      FLRFile <- paste(FPRESS.Home(),"\\outdata\\FLRObjects_",runref,"_",i,".dat",sep="")

      #check FLR file exists
      if (!(file.exists(FLRFile))) {
	 stop(paste("\nFLR data file",FLRFile,"doesn't exist\nUnable to continue\n",sep=" "))
      }

      load(FLRFile)
 
      ssb <- quantSums(NewSpawnMass)

      yrdat <- as.vector(ssb[,as.character(firstyear:finalyear),,,])

      belowssb1 <- 0
      belowssb2 <- 0

      for (j in 1:(years*iterations)) {

	 #first limit
	 if (yrdat[j] < ssbtest1) {
	    belowssb1 <- belowssb1+1
	 }

	 #second limit
	 if (yrdat[j] < ssbtest2) {
	    belowssb2 <- belowssb2+1
	 }

      }

      ssb1res <- c(ssb1res,(100.0*belowssb1)/(years*iterations))
      ssb2res <- c(ssb2res,(100.0*belowssb2)/(years*iterations))

   }

   dum <- seq(1,100,length=length(var))

   main.title <- paste("Risk of falling below SSB in",firstyear)

   if (years>1) {
      main.title <- paste("Risk of falling below SSB between",as.character(firstyear),"and",as.character(finalyear))
   }

   plot(var,dum,type="n",xlab=xlabel,ylab="%age Risk",main=main.title)

   lines(var,ssb1res,lty=1,lwd=1,type="o",pch="o")
   lines(var,ssb2res,lty=2,lwd=1,type="o",pch="o")

   xtext <- var

   ytext <- ssb1res+3
   text(xtext,ytext,format(ssb1res,digits=3),cex=0.5)
   
   ytext <- ssb2res+3
   text(xtext,ytext,format(ssb2res,digits=3),cex=0.5)

   leglabs <- c(paste("SSB =",ssbtest1),paste("SSB =",ssbtest2))
   leglty <- c(1,2)

   legend(legx,100,leglabs,lty=leglty,lwd=1,title=paste("options",runref,".xml",sep=""))

   par(def.par)
}
