#ssbvarplot.r
#Andrew Campbell
#Marine Institite, Galway
#andrew.campbell@marine.ie

###############################################
# Plot of SSB at end of projection versus F/TAC

ssbvarplot<-function(runref="missing",
		     box.whisker=1,
		     conf.interval=1){

   #runref - simulation reference
   #box.whisker - boxwhisker flag
   #conf.interval - confidence interval flag

   #store graphics settings
   def.par<-par(no.readonly=TRUE)

   #prompt for run reference
   if (missing(runref)) { 
      runref<-winDialogString("Run Reference?","")
   }

   if (is.null(runref)) {stop("Cancelled\n")}

   if (nchar(runref)==0) {stop("Exit - invalid run reference")}

   #options file
   optionsfile = paste(FPRESS.Home(),"\\options\\","options",runref,".xml",sep="")

   #check options file exists
   if (!(file.exists(optionsfile))) {
      stop(paste("\nOptions file",optionsfile,"doesn't exist\nUnable to continue\n",sep=" "))
   }

   #load xml tree
   doc<-xmlTreeParse(optionsfile)

   #root xmlNode object
   root<-xmlRoot(doc)

   #node values
   startyear <- as.integer(nodeval(root,"startyear","2000"))
   years <- as.integer(nodeval(root,"years",1))
   resolution <- as.integer(nodeval(root,"resolution",1))
   iterations <- as.integer(nodeval(root,"iterations",1))
   ssbtest1 <- as.integer(nodeval(root,"ssbtest1",0))
   ssbtest2 <- as.integer(nodeval(root,"ssbtest2",0))
   varmin <- as.numeric(nodeval(root,"varmin",0))
   varmax <- as.numeric(nodeval(root,"varmax",0))
   variable <- as.integer(nodeval(root,"variable",0))

   if (resolution <= 1) stop("invalid plot")

   ssbdata<-array(0,c(resolution+1,iterations + 3))

   for (i in 1:(resolution+1))
   {
      FLRFile = paste(FPRESS.Home(),"\\outdata\\FLRObjects_",runref,"_",i,".dat",sep="")  

      #check FLR file exists
      if (!(file.exists(FLRFile))) {
	 stop(paste("\nFLR data file",FLRFile,"doesn't exist\nUnable to continue\n",sep=" "))
      }

      load(FLRFile)

      ssbvals<-as.numeric(quantSums(NewSpawnMass[,years,,,]))

      meanssb<-mean(ssbvals)
      ssbcvs<-sqrt(var(ssbvals,y=NULL,na.rm=FALSE))/mean(ssbvals)

      if (variable==1) {
	 #calculate the fval
	 fval<-((varmax-varmin)*(i-1)/resolution)+varmin
	 #make sure we don't use an fval of 0
	 if(fval==0){fval<-0.0001}
	 temp<-fval
      } else {
	 #calculate the tacvalue
	 tacvalue<-((varmax-varmin)*(i-1)/resolution)+varmin
	 #make sure we don't use a tacvalue of 0
	 if(tacvalue==0){tacvalue<-0.0001}
	 temp<-tacvalue
      }
    
      ssbdata[i,]<-c(temp,meanssb,ssbcvs,ssbvals)
  
   }
  
   layout(matrix(c(1), 1, 1, byrow = TRUE))

   ftac<-ssbdata[,1]
   ssb<-ssbdata[,2]
   ssbcv<-ssbdata[,3]

   wid<-(max(ftac)-min(ftac))/(4*resolution)

   if(is.na(mean(ssbcv))==TRUE){ssbcv<-0} else{}

   sdum<-seq(0,1.3*max(ssb+1.96*ssb*ssbcv),length=length(ftac))

   if (variable==1) {
      plot(ftac,sdum,xlab=expression(bar(F)[]),ylab=paste("SSB at end of", startyear+years-1),main=optionsfile,type="n")
      }

   if (variable==0) {
      plot(ftac,sdum,xlab="TAC",ylab=paste("SSB at end of", startyear+years-1),main=optionsfile,type="n")
   }

   lines(ftac,ssb,lwd=2)

   if (box.whisker==1) {
      for(j in 1:length(ftac)) {
	 ssbvals<-c()
	 for(k in 4:length(ssbdata[1,])){
	    ssbvals<-c(ssbvals,ssbdata[j,k])
	 }
	 boxplot(ssbvals,range=0,boxwex=wid,add=TRUE,at=ftac[j])
      }
   }

  if(conf.interval==1){
    lines(ftac,ssb-1.96*ssbcv*ssb,lty=2)
    lines(ftac,ssb+1.96*ssbcv*ssb,lty=2)
    lines(ftac,ssb-2.58*ssbcv*ssb,lty=2)
    lines(ftac,ssb+2.58*ssbcv*ssb,lty=2)
  }

  bpa<-seq(ssbtest2,ssbtest2,length=length(ftac))
  blim<-seq(ssbtest1,ssbtest1,length=length(ftac))

  lines(ftac,bpa,lty=3)
  lines(ftac,blim,lty=3)
  
  #lin1<-seq(10000,10000,length=length(ftac))
  #lines(ftac,lin1,lty=4)

   #restore graphics
   par(def.par)

}
