#summaryplot.r
#Andrew Campbell
#Marine Institite, Galway
#andrew.campbell@marine.ie

####################################################################
# 4 plot of SSB / catch weight / recruits / F versus year.##########
####################################################################

summaryplot<-function(runref = "missing",
		      resref = "missing",
                      box.whisker = 1,
		      conf.int = 1,
		      ssbyminval = 0,
		      ssbymaxval = 0,
		      cwyminval = 0,
		      cwymaxval = 0,
		      fyminval = 0,
		      fymaxval = 0,
		      recyminval = 0,
		      recymaxval = 0){

   #runref - the run reference
   #resref - the TAC or f value for simulations with resolution > 0
   #box.whisker - boxwhisker flag
   #conf.int - confidence interval flag
   #ssbyminval - minimum y value for SSB vs Time plot
   #ssbymaxval - maximum y value for SSB vs Time plot
   #cwyminval - minimum y value for Catch Weight vs Time plot
   #cwymaxval - maximum y value for Catch Weight vs Time plot
   #fyminval - minimum y value for f vs Time plot
   #fymaxval - maximum y value for f vs Time plot
   #recyminval - minimum y value for recruitment vs Time plot
   #recymaxval - maximum y value for recruitment vs Time plot

   #prompt for run reference
   if (missing(runref)) { 
      runref<-winDialogString("Run Reference?","")
   }

   if (is.null(runref)) {stop("Cancelled\n")}

   if (nchar(runref)==0) stop("Exit")

   #construct options filename
   optionsfile = paste(FPRESS.Home(),"\\options\\","options",runref,".xml",sep="")

   #check options file exists
   if (!(file.exists(optionsfile))) {
      stop(paste("\nOptions file",optionsfile,"doesn't exist\nUnable to continue\n",sep=" "))
   }

   #read the relevant parameters for this run from the options file
   #load xml tree
   doc<-xmlTreeParse(optionsfile)

   #root xmlNode object
   root<-xmlRoot(doc)

   #node values
   startyear<-as.integer(nodeval(root,"startyear",1))
   years<-as.integer(nodeval(root,"years",1))
   variable<-as.integer(nodeval(root,"variable",0))
   resolution<-as.integer(nodeval(root,"resolution",1))
   iterations<-as.integer(nodeval(root,"iterations",1))
   fbarmin<-as.integer(nodeval(root,"fbarmin",1))
   fbarmax<-as.integer(nodeval(root,"fbarmax",1))
   ssbtest1<-as.integer(nodeval(root,"ssbtest1",1))
   ssbtest2<-as.integer(nodeval(root,"ssbtest2",1))
   rec<-as.integer(nodeval(root,"rec",1))
   seggrad<-as.numeric(nodeval(root,"seggrad",0))
   recrmean<-as.numeric(nodeval(root,"recmean",0))
   ssbcut<-as.numeric(nodeval(root,"ssbcut",0))
   hcrrule<-as.integer(nodeval(root,"hcrrule",0))
   varmin<-as.numeric(nodeval(root,"varmin",0))
   varmax<-as.numeric(nodeval(root,"varmax",0))

   res.interval <- (varmax-varmin)/resolution

   if (variable==0) {
      strVariable <- "TAC"
   } else {
      strVariable <- "F"
   }

   def.par<-par(no.readonly=TRUE)

   ftacvalue<-varmin

   if (resolution > 1) {

      #if resolution reference has not been supplied, prompt for it
      if (missing(resref)) {

         resref<-winDialogString(paste(strVariable," value? (min=",as.character(varmin),", max=",as.character(varmax),
						   ", interval = ",as.character(res.interval),")",sep=""),
						   as.character(varmin))

	 if (is.null(resref)) {stop("Cancelled\n")}

      } else {

	 #validate supplied value
	 if ((resref < varmin) || (resref > varmax)) {
	    stop("Invalid resolution reference\ncannot continue\n")
	 }
      }
      
      if (nchar(resref)==0) stop("Invalid resolution reference\ncannot continue\n")

      resref <- ((as.numeric(resref)-varmin)/res.interval)+1

      #if resref is too large, simply set it equal to value of resolution + 1, i.e. the maximum ftac value
      if (as.integer(resref) > resolution) {
	 resref <- resolution + 1
      }
      
      #calculate appropriate ftacvalue
      ftacvalue<-((varmax-varmin)*(as.integer(resref)-1)/resolution)+varmin

   } else {

      resref="1"
   
   }

   #construct data filename
   FLRFile = paste(FPRESS.Home(),"\\outdata\\FLRObjects_",runref,"_",resref,".dat",sep="")
 
   #check FLR file exists
   if (!(file.exists(FLRFile))) {
      stop(paste("\nFLR data file",FLRFile,"doesn't exist\nUnable to continue\n",sep=" "))
   }

   #load FLQuant objects
   load(FLRFile)

   #4 plots
   layout(matrix(c(1,2,3,4), 2, 2, byrow = TRUE))

   #vector of years
   year<-seq(startyear,length=years)

   #box width for box and whisker plots
   wid<-(max(year)-min(year))/(4*length(year))

   ###############################################################################################################
   #SSB PLOT
   ssb<-quantSums(NewSpawnMass)                               #ssb by year and iteration (all age groups summed) 
   ssbres<-sweep(ssb,2,apply(ssb,2,mean),"-")                 #ssb residuals
   ssbmean<-apply(ssb,2,mean)                                 #mean ssb (averaged over all iterations)
   ssbmedian<-apply(ssb,2,median)
   ssbstddev<-sqrt(areaSums(ssbres*ssbres)/(iterations-1))    #standard deviation
   ssbcv<-ssbstddev/ssbmean                                   #ssb coefficient of variation

   #Y axis limits 

   if (ssbymaxval==0) {
      if (max(ssbres,na.rm=FALSE)>0) {
	 ssbymax<-1.5*max((ssbmean+1.96*ssbmean*ssbcv),na.rm=TRUE)
      } else {
	 ssbymax<-max(ssbmean,na.rm=TRUE)
      }
   } else {
      ssbymax=ssbymaxval
   }

   if (ssbyminval==0) {
      ssbymin<-0
   } else {
      ssbymin=ssbyminval
   }

   #override the limits for specific simulations
   if (runref == "baseline01") {
      sdum<-seq(0,50000,length=length(year))   #baseline01
   } else if (runref == "baseline02") {
      sdum<-seq(0,25000,length=length(year))   #baseline02
   } else if (runref == "baseline03") {
      sdum<-seq(0,300000,length=length(year))  #baseline03
   } else {
      sdum<-seq(ssbymin,ssbymax,length=length(year))
   }
  
   #call plot with type n to create the graph area
   plot(year,sdum,xlab="Year",ylab="SSB (tonnes)",main="SSB",type="n")
   #lines(year,ssbmean,lwd=2)
   #draw median
   lines(year,ssbmedian,lwd=2)
   #add name of options file
   text(startyear+years/2,0,optionsfile,cex=0.65)

   #create the Bpa and Blim data sequences......
   bpa<-seq(ssbtest2,ssbtest2,length=length(year))
   blim<-seq(ssbtest1,ssbtest1,length=length(year))

   #....and plot them
   lines(year,bpa,lty=3)
   lines(year,blim,lty=3)
  
   #plot 95% & 99% confidence lines?
   if(conf.int==1){
      lines(year,ssbmean-1.96*ssbcv*ssbmean,lty=2)
      lines(year,ssbmean+1.96*ssbcv*ssbmean,lty=2)
      lines(year,ssbmean-2.58*ssbcv*ssbmean,lty=2)
      lines(year,ssbmean+2.58*ssbcv*ssbmean,lty=2)
   }

   #plot box whisker lines?
   if(box.whisker==1){
      for(i in 1:years){
	 ssbvals<-as.numeric(ssb[,i,,,])
	 boxplot(ssbvals,range=0,boxwex=wid,add=TRUE,at=i+startyear-1)
      }
   }


   ######################################################################################################
   #CATCH WEIGHT PLOT
   cw<-quantSums(CatchWeight)                               #catch weight by year and iteration (all age groups summed)
   cwres<-sweep(cw,2,apply(cw,2,mean),"-")                  #residuals
   cwmean<-apply(cw,2,mean)                                 #mean (averaged over all iterations)
   cwmedian<-apply(cw,2,median)                             #median
   cwstddev<-sqrt(areaSums(cwres*cwres)/(iterations-1))     #standard deviation
   cwcv<-cwstddev/cwmean                                    #coefficient of variation

   #Y axis limits
   if (cwymaxval==0) {
      if (max(cwres,na.rm=FALSE)>0) {
	 cwymax<-1.5*max((cwmean+1.96*cwmean*cwcv),na.rm=TRUE)
      } else {
	 cwymax<-max(cwmean,na.rm=TRUE)
      }
   } else {
      cwymax=cwymaxval
   }

   if (cwyminval==0) {
      cwymin<-0
   } else {
      cwymin=cwyminval
   }

   #y coordinates for text output
   ycoords<-seq(cwymin,cwymax,length.out=15)
  
   #simulation specific overrides
   if (runref == "baseline01") {
      cdum<-seq(0,30000,length=length(year))   #baseline01
   } else if (runref == "baseline02") {
      cdum<-seq(0,15000,length=length(year))   #baseline02
   } else if (runref == "baseline03") {
      cdum<-seq(0,25000,length=length(year))  #baseline03
   } else {
      cdum<-seq(cwymin,cwymax,length=length(year))    #default
   }

   #create plot
   plot(year,cdum,xlab="Year",ylab="Catch Weight (tonnes)",main="Catch Weight",type="n")
   #TAC value info text
   if (variable==0) {
      text(startyear,ycoords[15],paste("TAC Value",ftacvalue),cex=0.75,pos=4)
   } else {
      text(startyear,ycoords[15],paste("F Value",ftacvalue),cex=0.75,pos=4)
   }
   #HCR rule info text
   text(startyear,ycoords[14],paste("HCR Rule",hcrrule),cex=0.75,pos=4)
   
   #mean catch
   #lines(year,cwmean,lwd=1)
   #median catch
   lines(year,cwmedian,lwd=2)

   #95%/99% confidence limits
   if(conf.int==1){
      lines(year,cwmean-1.96*cwcv*cwmean,lty=2)
      lines(year,cwmean+1.96*cwcv*cwmean,lty=2)
      lines(year,cwmean-2.58*cwcv*cwmean,lty=2)
      lines(year,cwmean+2.58*cwcv*cwmean,lty=2)
   }

   #box-whisker plots?
   if(box.whisker==1){
      for(i in 1:years){
	 catvals<-as.numeric(cw[,i,,,])
	 boxplot(catvals,range=0,boxwex=wid,add=TRUE,at=i+startyear-1)
      }
   }


   #########################################################################################################
   #FISHING MORTALITY PLOT
   f<-FBar
   fmean<-apply(f,2,mean)                                          #mean (averaged over all iterations)
   fmedian<-apply(f,2,median)                                      #median
   fres<-round(sweep(f,2,apply(f,2,mean),"-"),8)                   #residuals
   fstddev<-sqrt(areaSums(fres*fres)/(iterations-1))               #standard deviation
   fcv<-fstddev/fmean                                              #coefficient of variation

   #Y axis limits
   if (fymaxval==0) {
      if (max(fres,na.rm=FALSE)>0) {
	 fymax<-1.5*max((fmean+1.96*fmean*fcv),na.rm=TRUE)
      } else {
	 fymax<-max(fmean,na.rm=TRUE)
      }
   } else {
      fymax=fymaxval
   }

   if (fyminval==0) {
      fymin<-0
   } else {
      fymin=fyminval
   }

   #simulation specific overrides
   if (runref == "baseline01") {
      fdum<-seq(0,3,length=length(year))   #baseline01
   } else if (runref == "baseline02") {
      fdum<-seq(0,1.5,length=length(year))   #baseline02
   } else if (runref == "baseline03") {
      fdum<-seq(0,3.5,length=length(year))  #baseline03
   } else {
      fdum<-seq(fymin,fymax,length=length(year))
   }

   #create plot
   plot(year,fdum,xlab="Year",ylab=expression(bar(F)[]),main=expression(paste(bar(F)[])),type="n")
   text(max(year)-((max(year)-startyear)*0.5),fymax,paste("(Average over ages", fbarmin, "-", fbarmax,")"),cex=0.75)
   #mean values
   #lines(year,fmean,lwd=2)
   #median values
   lines(year,fmedian,lwd=2)

   #95%/99% confidence intervals?
   if(conf.int==1){
      lines(year,fmean-1.96*fcv*fmean,lty=2)
      lines(year,fmean+1.96*fcv*fmean,lty=2)
      lines(year,fmean-2.58*fcv*fmean,lty=2)
      lines(year,fmean+2.58*fcv*fmean,lty=2)
   }

   #box-whisker lines
   if(box.whisker==1){
      for(i in 1:years){
	 fvals<-as.vector(f[,i,,,])
	 boxplot(fvals,range=0,boxwex=wid,add=TRUE,at=i+startyear-1)
	 #boxplot(fvals,range=1.5,boxwex=wid,add=TRUE,at=i+startyear-1)
      }
   }


   #######################################################################################################################
   #RECRUITMENT PLOT
   recmean<-areaMeans(Recruits)                              #mean (over iterations)
   recmedian<-apply(Recruits,2,median)                       #median
   recres<-sweep(Recruits,2,recmean,"-")                     #residuals
   recstddev<-sqrt(areaSums(recres*recres)/(iterations-1))   #standard deviation
   reccv<-recstddev/recmean                                  #coefficient of variation

   #Y-axis limits
   if (recymaxval==0) {
      if (max(recres,na.rm=FALSE)>0) {
	 recymax<-1.5*max((recmean+1.96*recmean*reccv),na.rm=TRUE)
      } else {
	 recymax<-max(recmean,na.rm=TRUE)
      }
   } else {
      recymax=recymaxval
   }

   if (recyminval==0) {
      recymin<-0
   } else {
      recymin=recyminval
   }

   #y coordinates for text output
   ycoords<-seq(recymin,recymax,length.out=15)

   #rdum<-seq(0,1.5*max(recmean+1.96*recmean*reccv),length=length(year))
  
   #simulation specific overrides
   if (runref == "baseline01") {
      rdum<-seq(0,16000,length=length(year))   #baseline01
   } else if (runref == "baseline02") {
      rdum<-seq(0,8000,length=length(year))   #baseline02
   } else if (runref == "baseline03") {
      rdum<-seq(0,2000000,length=length(year))  #baseline03
   } else {
      rdum<-seq(recymin,recymax,length=length(year))
   }

   #create plot
   plot(year,rdum,xlab="Year",ylab="Recruits (000s)",main="Recruitment",type="n")
   text(startyear,ycoords[15],paste("Recruitment Model",rec),cex=0.75,pos=4)
   
   #for segmented regression recruitment model, add info about the recruitment parameters
   if (rec==6) {
      text(startyear,ycoords[14],paste("Slope",seggrad),cex=0.75,pos=4)
      text(startyear,ycoords[13],paste("Breakpoint",ssbcut),cex=0.75,pos=4)
      text(startyear,ycoords[12],paste("Mean",recrmean),cex=0.75,pos=4)
   }
  
   #mean vals
   #lines(year,recmean,lwd=2)
   #median vals
   lines(year,recmedian,lwd=2)

   #95%/99% confidence intervals?
   if(conf.int==1){
      lines(year,recmean-1.96*reccv*recmean,lty=2)
      lines(year,recmean+1.96*reccv*recmean,lty=2)
      lines(year,recmean-2.58*reccv*recmean,lty=2)
      lines(year,recmean+2.58*reccv*recmean,lty=2)
   }

   #box whisker lines?
   if(box.whisker==1){
      for(i in 1:years){
	 recvals<-as.numeric(Recruits[,i,,,])
	 boxplot(recvals,range=0,boxwex=wid,add=TRUE,at=i+startyear-1)
	 #boxplot(recvals,range=0.1,boxwex=wid,add=TRUE,at=i+startyear-1)
      }
   }
   
   #reset defaults
   par(def.par)

}

#EOF#############################

