#yieldplot.r
#Andrew Campbell
#Marine Institite, Galway
#andrew.campbell@marine.ie

####################################################################
# Plots of average yield and variability in yield ##################
####################################################################

yieldplot<-function(runref="missing",
		    box.whisker=1){

   #store graphics settings
   def.par<-par(no.readonly=TRUE)

   #prompt for run reference
   if (missing(runref)) { 
      runref<-winDialogString("Run Reference?","")
   }

   if (is.null(runref)) {stop("Cancelled\n")}

   if (nchar(runref)==0) {stop("Exit - invalid run reference")}

   #construct options filename
   optionsfile = paste(FPRESS.Home(),"\\options\\options",runref,".xml",sep="")

   #check options file exists
   if (!(file.exists(optionsfile))) {
      stop(paste("\nOptions file",optionsfile,"doesn't exist\nUnable to continue\n",sep=" "))
   }

   #load xml tree
   doc<-xmlTreeParse(optionsfile)

   #root xmlNode object
   root<-xmlRoot(doc)

   #node values
   stockname <- nodeval(root,"stockname","")
   startyear <- as.integer(nodeval(root,"startyear","2000"))
   years <- as.integer(nodeval(root,"years",1))
   iterations <- as.integer(nodeval(root,"iterations",1))
   resolution <- as.integer(nodeval(root,"resolution",1))
   varmin <- as.numeric(nodeval(root,"varmin",0))
   varmax <- as.numeric(nodeval(root,"varmax",0))
   variable <- as.integer(nodeval(root,"variable",0))

   if (resolution==0) {stop("\nthis plot only available for simulations based on multiple ftac values\n")}

   #size of array dictated by the simulation type and the resolution
   yieldmeans<-array(0,c(resolution+1,iterations))
   yieldcvs<-array(0,c(resolution+1,iterations))

   #graph layout
   layout(matrix(c(1,2), 1, 2, byrow = TRUE))

   ftacmin <- varmin
   ftacmax <- varmax

   #ftac vector
   ftac<-seq(ftacmin,ftacmax,by=((ftacmax-ftacmin)/resolution))

   for(res in 1:(resolution+1)){

      FLRFile = paste(FPRESS.Home(),"\\outdata\\FLRObjects_",runref,"_",res,".dat",sep="")      

      #check FLR file exists
      if (!(file.exists(FLRFile))) {
	 stop(paste("\nFLR data file",FLRFile,"doesn't exist\nUnable to continue\n",sep=" "))
      }

      load(FLRFile)

      #yield is the sum of catches across all age groups
      annualyield<-quantSums(CatchWeight)			     #annual yields
      meanannualyield<-yearMeans(annualyield)			     #mean annual yields
      yieldres<-sweep(annualyield,5,meanannualyield,"-")	     #residuals
      yieldstddev<-sqrt(yearSums(yieldres*yieldres)/(iterations-1))  #standard deviation
      yieldcv<-yieldstddev/meanannualyield			     #coefficient of variation

      yieldmeans[res,]<-meanannualyield
      yieldcvs[res,]<-yieldcv

   }

   #width of bars for box whisker plots
   wid<-(max(ftac)-min(ftac))/(4*(resolution+1))

   #y axis vector
   sdum<-seq(0,1.1*max(yieldmeans),length=length(ftac))

   if(variable==1){xlabel=expression(bar(F)[])} else {xlabel="TAC"}

   #create plot
   plot(ftac,sdum,xlab=xlabel,ylab=paste("Mean yield from",startyear,"to",startyear+years),main="",type="n")

   temp<-c()

   for(j in 1:(resolution+1)){
      temp<-c(temp,mean(yieldmeans[j,]))
      if(box.whisker==1){
	 boxplot(yieldmeans[j,],range=0,boxwex=wid,add=TRUE,at=ftac[j])
      }
   }

   #mean yield
   lines(ftac,temp,lwd=2)

   #create plot for variability in yield
   cvdum<-seq(0,1.1*max(yieldcvs),length=length(ftac))
   plot(ftac,cvdum,xlab=xlabel,ylab=paste("Variability in yield over years", startyear,"to",startyear+years),main="",type="n")

   temp<-c()

   for(j in 1:(resolution+1)){
      temp<-c(temp,mean(yieldcvs[j,]))
      if(box.whisker==1){
	 boxplot(yieldcvs[j,],range=0,boxwex=wid,add=TRUE,at=ftac[j])
      }
   }

   #mean cv
   lines(ftac,temp,lwd=2)
   
   #restore graphics
   par(def.par)
}

# EOF
